local resourceShipCount = 1

local function newShip(player, item)
  local id = player:buyShip("shuttle", player.home, 0)
  local ship = game.world:getObjectData(id)
  ship.cargo.item = item
  ship.cargo.amt = ship.cargo.max
  table.insert(player.tradeShips, id)
  --dbg.add(item.." trader launched.", "TRADER "..player.id)
end

--MAIN CHECK BRANCH--
--Branch with leaves to check the AI has bought all his starting stuff
local haveShips = engine.AI.node.base:extend()
      function haveShips:update(player)
        if player.ships[1] == nil then
          player.tradeShips = {}
          for i = 1, resourceShipCount do
            for k, v in pairs(engine.items.resource) do
              newShip(player, k)
            end
            for k, v in pairs(engine.items.tier_1) do
              newShip(player, k)
            end
            for k, v in pairs(engine.items.tier_2) do
              newShip(player, k)
            end
            for k, v in pairs(engine.items.tier_3) do
              newShip(player, k)
            end
          end
          --dbg.add("Ships bought.", "TRADER "..player.id)
        end
        return true
      end

local check_branch = engine.AI.node.sequence:extend()
      function check_branch:init()
        self:add(haveShips:new())
      end

--TRADE BRANCH--
--Branch to control players traders.

local find_idle = engine.AI.node.base:extend()
      function find_idle:init()
        self.ticker = 0
      end
      function find_idle:update(player)
        self.ticker = self.ticker + 1
        if self.ticker > #player.ships then
          self.ticker = 0
          return false
        end
        local ship = game.world:getObjectData(player.ships[self.ticker])
        if ship.target.docked ~= 0 then
          --dbg.add("Checking player ship "..self.ticker, "TRADER "..player.id)
          --dbg.add("Ship state: "..ship.cargo.order, "TRADER "..player.id)
          if ship.cargo.order == "none" then
            player.aiShip = player.ships[self.ticker]
            --dbg.add("Ship "..player.aiShip.." idle.", "TRADER "..player.id)
            return true
          end
        end
        return "RUNNING"
      end

local check_buying = engine.AI.node.base:extend()
      function check_buying:update(player)
        local ship = game.world:getObjectData(player.aiShip)
        if ship.cargo.volume > 0 then
          local amt = ship.cargo.hold[ship.cargo.item].amt
          --dbg.add("Ship "..player.aiShip.." selling "..amt.." "..ship.cargo.item..".", "TRADER "..player.id)
          --ship.cargo.order = "sell"
          return false
        end
        --dbg.add("Ship "..player.aiShip.." looking for "..ship.cargo.item..".", "TRADER "..player.id)
        --ship.cargo.order = "buy"
        return true
      end

local find_sell_target = engine.AI.node.base:extend()
      function find_sell_target:update(player)
        local ship = game.world:getObjectData(player.aiShip)
        local targ = game.world:getObjectData(player.home)
        local planets = game.world:getObjectData(targ.localStar, "planets")
        local amt = ship.cargo.hold[ship.cargo.item].value
        local target = planets[1]
        for i = 1, #planets do
          local newTarget = game.world:getObjectData(planets[i], "colony")
          if newTarget.inventory[ship.cargo.item].price > amt and newTarget.id ~= ship.target.docked then
            amt = newTarget.inventory[ship.cargo.item].price
            target = planets[i]
          end
        end
        player:moveShipToPlanet(ship.id, target)
        ship.cargo.order = "sell"
        return true
      end

local find_buy_target = engine.AI.node.base:extend()
      function find_buy_target:update(player)
        local ship = game.world:getObjectData(player.aiShip)
        local targ = game.world:getObjectData(player.home)
        local planets = game.world:getObjectData(targ.localStar, "planets")
        local amt = math.huge
        local target = planets[1]
        local targAmt = 0
        --dbg.add("Checking Planets...", "TRADER "..player.id)
        for i = 1, #planets do
          local newTarget = game.world:getObjectData(planets[i], "colony")
          targAmt = newTarget.inventory[ship.cargo.item].price
          if targAmt < amt and newTarget.id ~= ship.target.docked then
            target = planets[i]
            amt = targAmt
          end
        end
        player:moveShipToPlanet(ship.id, target)
        ship.cargo.order = "buy"
        --dbg.add("Planet: "..target.." amt: "..targAmt, "TRADER "..player.id)
        return true
      end

local check_trade = engine.AI.node.sequence:extend()
      function check_trade:init()
        self:add(check_buying:new())
        self:add(find_buy_target:new())
      end

local buy_inverted = engine.AI.node.invert:extend()
      function buy_inverted:init()
        self:add(check_trade:new())
      end

local trade_branch = engine.AI.node.sequence:extend()
      function trade_branch:init()
        self:add(find_idle:new())
        self:add(buy_inverted:new())
        self:add(find_sell_target:new())
      end

local tree = engine.AI.node.loop:extend()
      function tree:init()
        self:add(check_branch:new())
        self:add(trade_branch:new())
      end

return tree