local texStexSizeize = engine.system.vars.planet.textureSize
local renderer = engine.system.behaviour:new()
      renderer.requirements = {
        "transform",
        "planet",
        }

function renderer:loadTextures(typ)
  if self.textures == nil then self.textures = {} end
  self.textures[typ] = {}
  for i = 1, engine.system.vars.planet.maxTextures do
    local map = engine.system.mapgen.generate(texSize, texSize)
    for k, v in pairs(engine.texture) do
      local label = string.sub(k, 1, #k - #typ - 1)
      local ext = string.sub(k, #k - #typ + 1, #k)
      if ext == typ then
        if self.textures[typ][label] == nil then self.textures[typ][label] = {} end
        table.insert(self.textures[typ][label], engine.system.mapgen.drawCanvas(map, v))
      end
    end
  end
end

function renderer:loadShade()
  self.textures.shade = {}
  for i = 1, 10 do
    table.insert(self.textures.shade, engine.system.mapgen.makeShadeLayer(texSize, texSize))
  end
end

function renderer:draw(data)
  local planet = data.planet
  local renderer = data.renderer
  local landImg = planet.landType
  local atmoImg = planet.atmoType
  local landId = planet.landId
  local atmoId = planet.atmoId
  local shadeId = planet.shadeId
  local canvas = planet.canvas
  local parentPos = game.world:getObjectData(data.orbiter.parentId, "transform")
  local starPos = game.world:getObjectData(data.localStar, "transform")
  local x = renderer.x
  local y = renderer.y
  local ss = renderer.screenScale
  local sc = data.transform.scale
  local texSize = engine.system.vars.planet.textureSize
  local scale = ss * sc
  local hWid = texSize / 2
  local hHig = texSize / 2
  local lr = math.rad(planet.landRot)
  local ar = math.rad(planet.atmoRot)
  local dx = data.transform.x - starPos.x
  local dy = data.transform.y - starPos.y
  local shadeAngle = math.deg(math.atan2(dy, dx))
  local lastCanvas = love.graphics.getCanvas()
  local orbitx, orbity = game.world.cameras[game.world.currentCamera]:getScreenPos(parentPos.x, parentPos.y)
  local offsetx, offsety = data.orbiter.offsetx * ss, -data.orbiter.offsety * ss
  love.graphics.setColor(128, 128, 128, 128)
  love.graphics.circle("line", orbitx + offsetx, orbity + offsety, data.orbiter.radius * ss, 360)
  love.graphics.setShader(engine.shaders[data.renderer.shader])
  engine.ui.colour.set("white")
  if landImg ~= nil then
    if landImg == "earth" and atmoImg ~= "oxygen" then
      data.planet.landType = "rock"
      landImg = "rock"
    end
    love.graphics.draw(self.textures.land[landImg][landId], x, y, lr, scale, scale, hWid, hHig)
  end
  if atmoImg ~= "none" then
    love.graphics.draw(self.textures.atmo[atmoImg][atmoId], x, y, ar, scale, scale, hWid, hHig)
  end
  love.graphics.draw(self.textures.shade[shadeId], x, y, math.rad(shadeAngle), scale, scale, hWid, hHig)
  love.graphics.setShader()
  if data.renderer.isSelected then
    local rad = (hWid + 10) * scale
    local offx = x + (rad)
    local offy = y + (rad)
    love.graphics.setColor(0, 255, 0, 255)
    love.graphics.circle("line", x, y, rad, 6)
    love.graphics.print("Type: "..data.planet.landType, offx, y)
    love.graphics.print("Atmo: "..data.planet.atmoType, offx, y + 17)
  end
  return data
end

return renderer