local renderer = engine.system.behaviour:new()
      renderer.requirements = {
        "transform",
        "renderer",
        }

local function lineToTarget(x, y, data)
  local target = game.world:getObjectData(data.target.id, "transform")
  local tx, ty = game.world.cameras[game.world.currentCamera]:getScreenPos(target.x, target.y)
  love.graphics.setColor(0, 255, 0, 255)
  love.graphics.line(x, y, tx, ty)
end

local function drawDebug(x, y, data)
  local time = math.floor(math.abs(data.target.range) / data.engine.speed)
  local secs = math.floor(time % 60)
  local mins = math.floor(time / 60)
  local rMins = math.floor(mins % 60)
  local hours = math.floor(mins / 60)
  local str = string.format("%02d:%02d:%02d", hours, rMins, secs)
  love.graphics.print(str, x - (font:getWidth(str) / 2), y + 10)
  str = math.floor(data.engine.speed).." Km/s"
  love.graphics.print(str, x - (font:getWidth(str) / 2), y + 27)
  str = math.ceil(data.target.range).."Mm"
  love.graphics.print(str, x - (font:getWidth(str) / 2), y + 44)
end

function renderer:draw(data)
  local x = data.renderer.x
  local y = data.renderer.y
  local ss = data.renderer.screenScale
  local sc = data.transform.scale
  local scale = ss * sc
  
  if data.target ~= nil then
    if data.renderer.isSelected and data.target.id ~= 0 then
      lineToTarget(x, y, data)
      drawDebug(x, y, data)
    end
  end
  
  if scale < .5 and data.type == "ship" then
    local ownerId = data.owner.id
    engine.ui.colour.set(game.world.players[ownerId].colour)
    love.graphics.draw(engine.assets[data.class.."_icon"], x, y, math.rad(0), .5, .5, 8, 8)
    if data.renderer.isSelected then
      love.graphics.circle("line", x, y, 8, 6)
    end
    return
  end
  
  local image = data.renderer.image
  local hWid = data.renderer.w / 2
  local hHig = data.renderer.h / 2
  local r = math.rad(data.transform.r)
  if data.renderer.shader ~= nil then
    data.renderer.step = data.renderer.step + data.renderer.stepInc
    engine.shaders[data.renderer.shader]:send("iGlobalTime", data.renderer.step)
    love.graphics.setShader(engine.shaders[data.renderer.shader])
  end
  love.graphics.setColor(255, 255, 255, 255)
  --engine.ui.colour.set("white")
  love.graphics.draw(engine.assets[image], x, y, r, scale, scale, hWid, hHig)
  if data.renderer.shader ~= nil then
    love.graphics.setShader()
  end
  
  if data.renderer.isSelected then
    love.graphics.setColor(0, 255, 0, 255)
    love.graphics.circle("line", x, y, (data.renderer.w + 10) / 2, 6)
  end
  
  return data
end

return renderer