local trade = engine.system.behaviour:new()
      trade.requirements = {
        "transform",
        "target",
        "cargo",
        "route",
      }

function trade:buyGoods(planetID, cargo, conditions)
  local colony = game.world:getObjectData(planetID, "colony")
  local price = colony.inventory[conditions.item].price
  local maxPrice = conditions.maxBuyPrice
  if price <= maxPrice then
    local amt = cargo.max * (conditions.maxBuyAmount / 100)
    if cargo.hold[conditions.item] ~= nil then
      amt = amt - cargo.hold[conditions.item].amt
    end
    if amt == 0 then
      cargo.order = "none"
      return
    end
    cargo.item = conditions.item
    cargo.order = "buy"
    cargo.amt = amt
  end
end

function trade:sellGoods(planetID, cargo, conditions)
  if cargo.hold[conditions.item] == nil then
    return
  end
  local colony = game.world:getObjectData(planetID, "colony")
  local price = colony.inventory[conditions.item].price
  local minPrice = conditions.minSellPrice
  if price >= minPrice then
    cargo.item = conditions.item
    cargo.order = "sell"
    cargo.amt = cargo.max * (conditions.maxSellAmount / 100)
    dbg.add(cargo.item, "SELL "..cargo.amt)
  else
    cargo.order = "none"
  end
end

function trade:update(dt, data)
  if data.route.current == "None" then return end
  local playerID = data.owner.id
  local route = game.world.players[playerID].routes[data.route.current]
  local planetID = route.planets[data.route.planetIndex]
  if data.route.hasSold == nil then
    data.route.hasSold = false
  end
  
  --If we're docked at the planet...
  if data.target.docked == planetID then
    --Check if we're idle...
    if data.cargo.order == "none" then
      --Get the item list for this planet...
      local routeItems = route.items[data.route.planetIndex]
      --If there isn't one for this planet then get the master list in position 1
      if routeItems == nil then
        routeItems = route.items[1]
      end
      --If that is still nil then return an error
      if routeItems == nil then
        dbg.add("ERROR: No master item list found.", "ROUTE")
        return
      end
      --If the current item isn't nil then
      if routeItems[data.route.itemIndex] ~= nil then
        --Check if we're buying or selling...
        if data.route.hasSold == false then
          --We're selling...
          self:sellGoods(planetID, data.cargo, routeItems[data.route.itemIndex])
        else
          --We're buying...
          self:buyGoods(planetID, data.cargo, routeItems[data.route.itemIndex])
        end
        --Move to the next item in the list...
        data.route.itemIndex = data.route.itemIndex + 1
      else
        --If we exceed the length of the list then
        if data.route.itemIndex > #routeItems then
          --If we are selling then start the Buy Phase
          if data.route.hasSold == false then
            --Toggle hasSold to indicate that we have gone once through the item table selling
            data.route.hasSold = true
          else
            --We have finished buying items so move to the next planet
            data.route.planetIndex = data.route.planetIndex + 1
            --If we're at the end of the planet list then loop round to 1
            if data.route.planetIndex > #route.planets then
              data.route.planetIndex = 1
            end
          end
          --Either way set the item index to 1 so we're at the start of the item list for the next Planet/Phase.
          data.route.itemIndex = 1
        end
      end
    end
  else --If we are not at the right planet then...
    --if are headed there then carry on...
    if data.target.id == planetID then
      return
    end
    --Otherwise set a course for the planet....
    game.world.players[CURRENT_PLAYER]:moveShipToPlanet(data.id, planetID)--Engage :P
    data.cargo.order = "none"
    data.route.hasSold = false
  end
end

return trade