local ship_trade = engine.system.behaviour:new()
      ship_trade.requirements = {
        "target",
        "cargo",
        "owner",
      }

local function inventoryHasGoods(inv, item, amt)
  if inv[item] == nil then
    dbg.add(item, "ERROR")
    return 0
  end
  if inv[item].amt >= amt then
    return amt
  else
    return math.floor(inv[item].amt)
  end
end

local function inventoryHasSpace(inv, item, amt)
  if inv[item].amt + amt <= inv[item].max then
    return amt
  else
    return inv[item].max - inv[item].amt
  end
end

local function shipHasGoods(cargo, item, amt)
  if cargo.hold[item] == nil then return 0 end
  if cargo.hold[item].amt >= amt then
    return amt
  else
    return cargo.hold[item].amt
  end
end

local function shipHasSpace(cargo, item, amt)
  local data = engine.items.getData(item)
  local vol = data.volume * amt
  if cargo.volume + vol <= cargo.max then
    return amt
  else
    local space = cargo.max - cargo.volume
    return math.floor(space / data.volume)
  end
end

local function addCargo(cargo, item, amt, cost)
  local data = engine.items.getData(item)
  if cargo.hold[item] == nil then
    cargo.hold[item] = {
      amt = amt,
      cost = cost,
      value = cost / amt
    }
  else
    cargo.hold[item].amt = cargo.hold[item].amt + amt
    cargo.hold[item].cost = cargo.hold[item].cost + cost
    cargo.hold[item].value = cargo.hold[item].cost / cargo.hold[item].amt
  end
  cargo.volume = cargo.volume + (data.volume * amt)
  cargo.inventoryChanged = true
end

local function removeCargo(cargo, item, amt)
  if cargo.hold[item] == nil then return end
  local data = engine.items.getData(item)
  cargo.hold[item].amt = cargo.hold[item].amt - amt
  if cargo.hold[item].amt <= 0 then
    cargo.hold[item] = nil
  end
  cargo.volume = cargo.volume - (data.volume * amt)
  cargo.inventoryChanged = true
end

local function buyGoods(data, id, item, amt)
  local colony = game.world:getObjectData(id, "colony")
  local inv = colony.inventory
  local availableItems = inventoryHasGoods(inv, item, amt)
  local availableSpace = shipHasSpace(data.cargo, item, amt)
  if availableItems == 0 then
    data.cargo.order = "none"
    return
  end
  if availableSpace == 0 then
    data.cargo.order = "none"
    return
  end
  if amt > availableItems then
    amt = availableItems
  end
  if amt > availableSpace then
    amt = availableSpace
  end
  local cost = math.floor(inv[item].price * amt)
  --dbg.add(amt.." "..item.." from Planet "..id..".", "TRADER "..data.owner.id)
  if cost > game.world.players[data.owner.id].cash then
    amt = math.floor(game.world.players[data.owner.id].cash / inv[item].price)
    cost = math.floor(inv[item].price * amt)
  end
  if game.world.players[data.owner.id].cash >= cost and amt > 0 then
    game.world.players[data.owner.id].cash = game.world.players[data.owner.id].cash - cost
    game.world.objects[id].colony.inventory[item].amt = game.world.objects[id].colony.inventory[item].amt - amt
    engine.behaviour.colony_prices:update(.001, game.world.objects[id], true)
    addCargo(data.cargo, item, amt, cost)
  end
  data.cargo.order = "none"
end

local function sellGoods(data, id, item, amt)
  local colony = game.world:getObjectData(id, "colony")
  local inv = colony.inventory
  local availableSpace = inventoryHasSpace(inv, item, amt)
  local availableItems = shipHasGoods(data.cargo, item, amt)
  if availableItems == 0 then
    data.cargo.order = "none"
    return
  end
  if availableSpace == 0 then
    data.cargo.order = "none"
    return
  end
  if amt > availableItems then
    amt = availableItems
  end
  if amt > availableSpace then
    amt = availableSpace
  end
  local price = math.ceil(inv[item].price * amt)
  removeCargo(data.cargo, item, amt)
  colony.inventory[item].amt = colony.inventory[item].amt + amt
  engine.behaviour.colony_prices:update(.001, game.world.objects[id], true)
  game.world.players[data.owner.id].cash = game.world.players[data.owner.id].cash + price
  data.cargo.order = "none"
end

function ship_trade:update(dt, data)
  if data.cargo.order == "none" then return end
  if data.target.docked == 0 then return end
  local id = data.target.docked
  local order = data.cargo.order
  local item = data.cargo.item
  local amt = data.cargo.amt
  if order == "buy" then
    buyGoods(data, id, item, amt)
  elseif order == "sell" then
    sellGoods(data, id, item, amt)
  end
end

return ship_trade