local minDist = 1000
local test = {
      menuLabel = "New Game",
      timer = 10,
      timerMax = 25,
      sites = {},
      }

function test.startup()
  test.ui = engine.ui.window_manager:new()
  test.ui:add("menu", engine.prefab.menu_bar:new())
end

function test.switchShips(inc)
  local shipList = game.world.players[CURRENT_PLAYER].ships
  local currentShip = game.world.players[CURRENT_PLAYER].currentShip + inc or 1
  if currentShip < 1 then
    currentShip = #shipList
  elseif currentShip > #shipList then
    currentShip = 1
  end
  game.world.players[game.currentPlayer].currentShip = currentShip
  game.world.cameras.main:setTarget(shipList[currentShip])
end

function test.keyPressed(k, c, isRep)
  if test.ui:keyPressed(k, c, isRep) then return end
  local keys = engine.system.vars.hotkeys
  if k == "escape" then
    engine.state.set("game_menu")
  elseif k == keys.speedUp then
    game.world.speed = game.world.speed + 1
  elseif k == keys.speedDown then
    game.world.speed = math.max(game.world.speed - 1, 0)
  elseif k == keys.nextEntity then
    local lastStar = game.world.stars[#game.world.stars]
    local lastPlanet = game.world.objects[lastStar].planets[#game.world.objects[lastStar].planets]
    local target = game.world.cameras.main.target + 1
    if target > lastPlanet then
      target = 1
    end
    game.world.cameras.main:setTarget(target)
  elseif k == keys.lastEntity then
    local lastStar = game.world.stars[#game.world.stars]
    local lastPlanet = game.world.objects[lastStar].planets[#game.world.objects[lastStar].planets]
    local target = game.world.cameras.main.target - 1
    if target < 1 then
      target = lastPlanet
    end
    game.world.cameras.main:setTarget(target)
  elseif k == keys.nextPlayerShip then
    test.switchShips(1)
  elseif k == keys.lastPlayerShip then
    test.switchShips(-1)
  elseif k == keys.showShipWindow then
    if test.ui.objects["ship_"..CURRENT_SHIP] == nil then
      test.ui:add("ship_"..CURRENT_SHIP, engine.window.ship_window:new(), CURRENT_SHIP)
    else
      test.ui.objects["ship_"..CURRENT_SHIP]:toggleVisible()
    end
  elseif k == keys.zoomIn then
    local sc = game.world.cameras.main.scale
    local s = (sc / 2)
    game.world.cameras.main:setScale(math.min(sc + s, 50))
  elseif k == keys.zoomOut then
    local sc = game.world.cameras.main.scale
    local s = (sc / 2) * -1
    game.world.cameras.main:setScale(math.max(sc + s, .00001))
  end
end

function test.textInput(t)
  if test.ui:textInput(t) then return end
end

function test.checkRightClickOptions(id)
  if id ~= nil then
    if game.world.objects[id].type == "planet" or game.world.objects[id].type == "moon" then
      game.world.players[CURRENT_PLAYER]:assignDockOrder(id)
    end
  end
end

function test.checkLeftClickOptions(id)
  if test.ui ~= nil and id ~= nil then
    if game.world.objects[id].type == "planet" or game.world.objects[id].type == "moon" then
      local key = "planet_"..id
      if test.ui:windowExists(key) then
        test.ui:selectWindow(key)
      else
        test.ui:add(key, engine.window.planet_window:new(), id)
      end
    end
  end
end

function test.mousePressed(x, y, b)
  if test.ui:mousePressed(x, y, b) then return end
  if b == 1 then
    local c = game.world:checkClick(x, y, b)
    if not c then
      game.world:deselectEntities(CURRENT_PLAYER)
      game.world.mapx, game.world.mapy = game.world.cameras.main:getMapPos(x, y)
      game.world.dragSelect = true
    else
      game.world:deselectEntities(CURRENT_PLAYER)
      game.world.objects[c].renderer.isSelected = true
    end
  elseif b == 2 then
  end
end

function test.mouseReleased(x, y, b)
  if test.ui:mouseReleased(x, y, b) then return end
  if b == 1 then
    if game.world.mapx ~= nil then
      local minx = math.min(game.world.mapx, game.world.mapx2)
      local maxx = math.max(game.world.mapx, game.world.mapx2)
      local miny = math.min(game.world.mapy, game.world.mapy2)
      local maxy = math.max(game.world.mapy, game.world.mapy2)
      game.world:selectShips(CURRENT_PLAYER, minx, miny, maxx, maxy)
      game.world.dragSelect = false
      game.world.mapx = nil
    else
      test.checkLeftClickOptions(game.world:checkClick(x, y, b))
    end
  elseif b == 2 then
    test.checkRightClickOptions(game.world:checkClick(x, y, b))
  end
end

function test.wheelMoved(...)
  if test.ui:wheelMoved(...) then return end
  if game.world:wheelMoved(...) then return end
end

function test.resize(w, h)
  game.world.cameras.main:setSize(w, h)
end

function test.update(dt)
  test.ui:updateMaster(dt)
  game.world:update(dt)
end

function test.drawPlayerData()
  local balance = "Current Balance: Cr "..game.world.players[CURRENT_PLAYER].cash
  local x = math.floor((engine.screenWidth - font:getWidth(balance)) / 2)
  local y = 2
  love.graphics.print(balance, x, y)
end

function test.drawAICash()
  for i = 2, #game.world.players do
    local amt = game.world.players[i].cash or 1
    local str = "Player "..i..": "..amt
    local x = math.floor((engine.screenWidth - font:getWidth(str)))
    local y = ((i - 1) * 17) - 17
    love.graphics.print(str, x, y)
  end
end

function test.draw()
  game.world:draw()
  test.ui:draw()
  engine.ui.colour.set("green")
  test.drawPlayerData()
  test.drawAICash()
end

return test