local textureCount = 0
local textureType = "land"
local shadeCount = 0
local shadeMax = 10
local simLength = 200
local simCount = 0
local printStr = ""
local boxMax = engine.system.vars.planet.maxTextures
local texSize = engine.system.vars.planet.textureSize
local boxCurrent = 0
local boxWidth = 200
local boxHeight = 18
local worldSimulated = false
local maxAI = 3
local colours = {
  {0,200,0},
  {0,200,200},
  {0,0,200},
  {200,200,0},
  {200,0,200},
}

local loadGame = {}

local function loadTextures(tex)
  if textureCount == engine.system.vars.planet.maxTextures then return true end
  textureCount = textureCount + 1
  boxCurrent = textureCount
  if engine.behaviour.planet_renderer.textures == nil then engine.behaviour.planet_renderer.textures = {} end
  if engine.behaviour.planet_renderer.textures[tex] == nil then engine.behaviour.planet_renderer.textures[tex] = {} end
  local map = engine.system.mapgen.generate(texSize, texSize)
  for k, v in pairs(engine.texture) do
    local label = string.sub(k, 1, #k - #tex - 1)
    local ext = string.sub(k, #k - #tex + 1, #k)
    if ext == tex then
      if engine.behaviour.planet_renderer.textures[tex][label] == nil then engine.behaviour.planet_renderer.textures[tex][label] = {} end
      table.insert(engine.behaviour.planet_renderer.textures[tex][label], engine.system.mapgen.drawCanvas(map, v))
    end
  end
  printStr = "Loading Planet "..textureType.." texture "..textureCount.."/"..engine.system.vars.planet.maxTextures.."."
end

local function loadShade()
  shadeCount = shadeCount + 1
  boxCurrent = shadeCount
  if engine.behaviour.planet_renderer.textures.shade == nil then engine.behaviour.planet_renderer.textures.shade = {} end
  table.insert(engine.behaviour.planet_renderer.textures.shade, engine.system.mapgen.makeShadeLayer(texSize, texSize))
  printStr = "Loading Planet shade texture "..shadeCount.."/"..shadeMax.."."
end

local function checkTextureLoad()
  if textureType == "done" then return true end
  if textureType == "land" then
    if textureCount < engine.system.vars.planet.maxTextures then
      loadTextures("land")
    else
      textureCount = 0
      boxCurrent = 0
      textureType = "atmo"
    end
  elseif textureType == "atmo" then
    if textureCount < engine.system.vars.planet.maxTextures then
      loadTextures("atmo")
    else
      boxCurrent = 0
      boxMax = shadeMax
      textureType = "shade"
    end
  elseif textureType == "shade" then
    if shadeCount < shadeMax then
      loadShade()
    else
      boxCurrent = 0
      boxMax = simLength
      textureType = "done"
    end
  end
  return false
end

local function simulateWorld()
  if simCount == simLength then return true end
  simCount = simCount + 1
  boxCurrent = simCount
  game.world:update(10)
  printStr = "Simulating World..."
  return false
end

function loadGame.enter()
  
  engine.music[engine.currentSong]:setLooping(true)
  engine.music[engine.currentSong]:setVolume(.2)
  engine.music[engine.currentSong]:play()
  
  love.graphics.setDefaultFilter("linear", "linear", 0)
  
  if game.world == nil then
    game.world = engine.system.world:new()
    engine.system.sysgen.newSolaris(0, 0)
    for i = 2, #game.world.objects do
      game.world:addComponent(i, "colony", i, game.world.objects[i].transform.scale)
    end
      
    CURRENT_PLAYER = game.world:addPlayer("Player", 5000, 5, "none")
    game.currentPlayer = CURRENT_PLAYER
    game.world.players[CURRENT_PLAYER]:buyShip("shuttle", 5, 0)
    local shipId = game.world.players[CURRENT_PLAYER].currentShip
    CURRENT_SHIP = game.world.players[CURRENT_PLAYER].ships[shipId]
    game.world.cameras.main.target = shipId
    
    for i = 2, maxAI do
      local id = game.world:addPlayer("AI", 500000, i, "trader")
      game.world.players[id]:setColour(colours[i][1],colours[i][2],colours[i][3])
    end
  else
    worldSimulated = true
  end
  CURRENT_PLAYER = 1
  game.currentPlayer = 1
  CURRENT_SHIP = game.world.players[CURRENT_PLAYER].currentShip
  game.world.cameras.main.target = CURRENT_SHIP
  
  engine.commands.save("autosave")
  
end

function loadGame.update(dt)
  local gameLoaded = false
  if worldSimulated then
    if checkTextureLoad() then
      gameLoaded = true
    end
  else
    if checkTextureLoad() and simulateWorld() then
      gameLoaded = true
    end
  end
  if gameLoaded then
    engine.music[engine.currentSong]:stop()
    engine.currentSong = "Frozen_Star"
    engine.music[engine.currentSong]:setLooping(true)
    engine.music[engine.currentSong]:setVolume(.2)
    engine.music[engine.currentSong]:play()
    engine.state.set("game")
  end
end

function loadGame.draw()
  love.graphics.setColor(engine.system.vars.user.uiColour)
  local x = math.floor((engine.screenWidth - font:getWidth(printStr)) / 2)
  local y = math.floor((engine.screenHeight - font:getHeight()) / 2)
  love.graphics.print(printStr, x, y)
  
  local bx = math.floor((engine.screenWidth - boxWidth) / 2)
  local by = math.floor((engine.screenHeight - boxHeight) / 2) + (font:getHeight() + 2)
  love.graphics.rectangle("line", bx, by, boxWidth, boxHeight, 5, boxHeight)
  
  local perc = (boxCurrent / boxMax)
  local fWid = boxWidth * perc
  love.graphics.rectangle("fill", bx, by, fWid, boxHeight, 5, boxHeight)
end

return loadGame