local labelWidth = 128
local itemWidth = 38

local function newNameLabel(l, x, y, w, h)
  local tCol = engine.system.vars.user.uiText
  local label = engine.ui.button:extend()
        label:setTileDimensions(32, 16, 96, 16)
        function label:setup()
          if self.box.hasScrollBar then
            self:setSize(self.w - 32, self.h)
          end
          self.highlight = h
        end
        function label:update(dt)
          if self.highlight then
            self.isHovered = true
          end
        end
  return label:new(l, x, y, w, 16, "standard_spreadsheet_tile", "standard_spreadsheet_tile_hover", tCol, true)
end

local function newLabel(l, x, y, w, h, index)
  local tCol = engine.system.vars.user.uiText
  local label = engine.ui.button:extend()
        label:setTileDimensions(32, 16, 96, 16)
        function label:setup()
          if self.box.hasScrollBar then
            self:setSize(self.w - 32, self.h)
          end
          self.index = index
          if not h then
            self.align = "left"
          end
          self.index = index
        end
        function label:update(dt)
          if self.win.shipIndex == self.index then
            self.isHovered = true
          end
        end
        function label:onClick(x, y, b)
          self.win.shipIndex = self.index
        end
  return label:new(l, x, y, w, 16, "standard_spreadsheet_tile", "standard_spreadsheet_tile_hover", tCol, true)
end

local function newShipList(w, h)
  local list = engine.ui.container:extend()
        function list:fillList()
        local shipTab = {}
        for k, v in pairs(engine.ships) do
          table.insert(shipTab, k)
        end
        table.sort(shipTab)
        local x, y = 5, 5
        self:add("name", newNameLabel("Ship", x, y, labelWidth, true))
        self:add("cost", newNameLabel("Cost", self:stackRight("name", 0), y, itemWidth, true))
        self:add("acc", newNameLabel("Acc", self:stackRight("cost", 0), y, itemWidth, true))
        self:add("hold", newNameLabel("Hold", self:stackRight("acc", 0), y, itemWidth, true))
        y = self:stackUnder("name", 0)
        for i = 1, #shipTab do
          local template = engine.ships[shipTab[i]].new()
          x = 5
          self:add(shipTab[i], newLabel(template.name, x, y, labelWidth, false, shipTab[i]))
          x = self:stackRight(shipTab[i], 0)
          self:add(shipTab[i].."_cost", newLabel((template.cost / 1000000).."M", x, y, itemWidth, false, shipTab[i]))
          x = self:stackRight(shipTab[i].."_cost", 0)
          self:add(shipTab[i].."_acc", newLabel(template.engine.acc, x, y, itemWidth, false, shipTab[i]))
          x = self:stackRight(shipTab[i].."_acc", 0)
          self:add(shipTab[i].."_hold", newLabel(template.cargo.max, x, y, itemWidth, false, shipTab[i]))
          y = self:stackUnder(shipTab[i], 0)
        end
      end
      function list:setup()
        self:fillList()
      end
  return list:new("ships", 5, 5, w, h, "standard_container", "standard_container", {0,0,0,0}, true)
end

local function newBuyButton(y, w)
  local tCol = engine.system.vars.user.uiText
  local btn = engine.ui.button:extend()
        function btn:onClick(x, y, b)
          if self.win.shipIndex ~= nil then
            local template = engine.ships[self.win.shipIndex].new()
            game.world.players[CURRENT_PLAYER]:buyShip(self.win.shipIndex, self.win.planetId, template.cost)
          end
        end
  return btn:new("Buy", 5, y, w, 32, "standard_button", "standard_button_hover", tCol, true)
end

local sy = engine.ui.container:extend()

function sy:setup()
  self.win.shipIndex = "shuttle"
  self:add("list", newShipList(self.w - 10, self.h - 44))
  self:add("buy", newBuyButton(self:stackUnder("list", 2), self.w - 10))
end

return sy