local WIDTH = 437
local HEIGHT = 448
local TEXTCOL = engine.system.vars.user.userText
local BTNIMG = "standard_spreadsheet_tile"
local LEFTPAD = 7
local YPAD = 12
local BTNWIDTH = 64
local BTNHEIGHT = 16
local LABELWIDTH = 2.5

local tab_box = engine.ui.container:extend()

function tab_box:newBuyButton(x, y, w, h, action, amount)
  local label = string.upper(string.sub(action, 1, 1))..string.sub(action, 2)
  local b = engine.ui.button:extend()
        function b:onClick()
          local id = CURRENT_SHIP
          if game.world.objects[id].target.docked == self.win.planetId then
            game.world.objects[id].cargo.order = action
            game.world.objects[id].cargo.item = self.box.tradeItem
            game.world.objects[id].cargo.amt = amount
            dbg.add(label.."ing "..amount.." "..self.box.tradeItem..".", "TRADE")
          end
        end
  return b:new(label.." "..amount, x, y, w, h, "standard_button", "standard_button_hover", TEXTCOL, true)
end

function tab_box:newButton(x, y, id, comp, key, watch, altValue)
  local b = engine.ui.data_tracker:extend()
        b:setTracker(game.world.objects[id].colony.inventory, 0, comp, key)
        b:setAltValue(watch, altValue)
        b:setTileDimensions(32, 16, 96, 16)
        function b:onClick(x, y, b)
          self.box.tradeItem = comp
        end
  return b:new(comp, x, y, BTNWIDTH, BTNHEIGHT, BTNIMG, BTNIMG.."_hover", TEXTCOL, true)
end

function tab_box:newHeader(x, y, w, label)
  local b = engine.ui.button:new(label, x, y, w, BTNHEIGHT, BTNIMG.."_hover", BTNIMG.."_hover", TEXTCOL, true)
        b:setTileDimensions(32, 16, 96, 16)
  return b
end

function tab_box:newLabel(x, y, comp, key)
  local b = engine.ui.button:new(engine.items[comp][key].label, x, y, (BTNWIDTH * LABELWIDTH), BTNHEIGHT, BTNIMG, BTNIMG.."_hover", TEXTCOL, true)
        b:setTileDimensions(32, 16, 96, 16)
        function b:onClick(x, y, b)
          self.box.tradeItem = key
        end
  return b
end

function tab_box:addHeaders(y)
  local x = LEFTPAD
  self:add("name", self:newHeader(x, y, (BTNWIDTH * LABELWIDTH), "Item"))
  x = x + (BTNWIDTH * LABELWIDTH)
  self:add("amt", self:newHeader(x, y, BTNWIDTH, "Amount"))
  x = x + BTNWIDTH
  self:add("produced", self:newHeader(x, y, BTNWIDTH, "Produced"))
  x = x + BTNWIDTH
  self:add("used", self:newHeader(x, y, BTNWIDTH, "Used"))
  x = x + BTNWIDTH
  self:add("price", self:newHeader(x, y, BTNWIDTH, "$ x 1"))
end

function tab_box:addRow(y, id, comp, key)
  local x = LEFTPAD
  self:add(key.."_name", self:newLabel(x, y, comp, key))
  x = x + (BTNWIDTH * LABELWIDTH)
  self:add(key.."_amt", self:newButton(x, y, id, key, "amt", 0, "None"))
  x = x + BTNWIDTH
  self:add(key.."_prodString", self:newButton(x, y, id, key, "prodString", 0, "None"))
  x = x + BTNWIDTH
  self:add(key.."_usageString", self:newButton(x, y, id, key, "usageString", 0, "None"))
  x = x + BTNWIDTH
  self:add(key.."_cost", self:newButton(x, y, id, key, "price", 0, "None"))
end

function tab_box:setup(planetId)
  local x = LEFTPAD + 1
  local y = YPAD
  local wid = math.floor((self.w - ((LEFTPAD * 2) + 3)) / 4)
  self:addHeaders(y)
  local itemList = engine.items.getSortedTierList(self.label)
  for i = 1, #itemList do
    y = y + BTNHEIGHT
    self:addRow(y, self.win.planetId, self.label, itemList[i])
  end
  self.tradeItem = itemList[1]
  y = y + BTNHEIGHT + 2
  local sy = y + 33
  self:add("buy_1", self:newBuyButton(x, y, wid, 32, "buy", 1))
  self:add("sell_1", self:newBuyButton(x, sy, wid, 32, "sell", 1))
  x = x + wid + 1
  self:add("buy_10", self:newBuyButton(x, y, wid, 32, "buy", 10))
  self:add("sell_10", self:newBuyButton(x, sy, wid, 32, "sell", 10))
  x = x + wid + 1
  self:add("buy_50", self:newBuyButton(x, y, wid, 32, "buy", 50))
  self:add("sell_50", self:newBuyButton(x, sy, wid, 32, "sell", 50))
  x = x + wid + 1
  self:add("buy_100", self:newBuyButton(x, y, wid, 32, "buy", 100))
  self:add("sell_100", self:newBuyButton(x, sy, wid, 32, "sell", 100))
  
  self.isDocked = true
end

function tab_box:showButtons(bool)
  self:redrawObjects()
  self.objects["buy_1"].isVisible = bool
  self.objects["buy_10"].isVisible = bool
  self.objects["buy_50"].isVisible = bool
  self.objects["buy_100"].isVisible = bool
  self.objects["sell_1"].isVisible = bool
  self.objects["sell_10"].isVisible = bool
  self.objects["sell_50"].isVisible = bool
  self.objects["sell_100"].isVisible = bool
end

function tab_box:update(dt)
  
  self:deselectAll()
  self.objects[self.tradeItem.."_name"].isSelected = true
  self.objects[self.tradeItem.."_amt"].isSelected = true
  self.objects[self.tradeItem.."_prodString"].isSelected = true
  self.objects[self.tradeItem.."_usageString"].isSelected = true
  self.objects[self.tradeItem.."_cost"].isSelected = true
  
  self.wasDocked = self.isDocked
  self.isDocked = false
  if game.world.objects[CURRENT_SHIP].target.docked == self.win.planetId then
    self.isDocked = true
  end
  if self.wasDocked ~= self.isDocked then
    self:showButtons(self.isDocked)
    self.isDirty = true
  end
  
end

local box = engine.ui.container:extend()

function box:newTab(label, id)
  local y = YPAD
  local t = tab_box:extend()
        for k, v in pairs(engine.items[label]) do
          t:addRow(y, id, label, k)
          y = y + BTNHEIGHT
        end
  return t
end

function box:setup(planetId)
  self:add("trade_box", engine.ui.tab:new("trade_Tabs", 0, 0, WIDTH, HEIGHT, "standard_container", "standard_container_hover", TEXTCOL, true))
  self.objects.trade_box:addTab("resource", tab_box)
  self.objects.trade_box:addTab("tier_1", tab_box)
  self.objects.trade_box:addTab("tier_2", tab_box)
  self.objects.trade_box:addTab("tier_3", tab_box)
end

return box