local route = {}

function route.startup()
  route.currentName = "New_Route"
  route.ui = engine.ui.window_manager:new()
  route.ui:add("menu", engine.prefab.menu_bar:new())
  route.ui:add("helper", engine.window.route_helper:new())
  route.ui:add("current", engine.window.route_window:new())
end

function route.enter()
  route.lastSpeed = game.world.speed
  route.lastZoom = game.world.cameras.main.scale
  route.lastTarget = game.world.cameras.main.target
  if route.lastTarget == 0 then
    route.lastX = game.world.cameras.main.x
    route.lastY = game.world.cameras.main.y
  end
  route.currentList = nil
  route.itemList = nil
  route.currentName = "New_Route"
  route.selectedPlanet = 1
  route.refreshLists = true
  game.world.cameras.main.scale = 0.0001
  game.world.speed = 0
  
  
  engine.music[engine.currentSong]:stop()
  engine.currentSong = "Sneaky_Adventure"
  engine.music[engine.currentSong]:setLooping(true)
  engine.music[engine.currentSong]:setVolume(.2)
  engine.music[engine.currentSong]:play()
end

function route.exit()
  game.world.cameras.main:setScale(route.lastZoom)
  if route.lastTarget ~= 0 then
    game.world.cameras.main:setTarget(route.lastTarget)
  else
    game.world.cameras.main:setPosition(route.lastX, route.lastY)
  end
  game.world.speed = route.lastSpeed
  
  engine.music[engine.currentSong]:stop()
  engine.currentSong = "Frozen_Star"
  engine.music[engine.currentSong]:setLooping(true)
  engine.music[engine.currentSong]:setVolume(.2)
  engine.music[engine.currentSong]:play()
end

function route.loadRoute(key)
  local tradeData = {}
        tradeData.planets = {}
        for i = 1, #game.world.players[CURRENT_PLAYER].routes[key].planets do
          tradeData.planets[i] = game.world.players[CURRENT_PLAYER].routes[key].planets[i]
        end
        tradeData.items = {}
        for i = 1, #game.world.players[CURRENT_PLAYER].routes[key].planets do
          tradeData.items[i] = game.world.players[CURRENT_PLAYER].routes[key].items[i]
        end
  if tradeData == nil then
    dbg.add("NO TRADE ROUTE BY THIS NAME..."..key, "ERROR")
  else
    route.currentName = key
    route.currentList = tradeData.planets
    route.itemList = tradeData.items
    route.selectedItem = 1
    route.refreshLists = true
    route.ui:closeWindow("load_window")
  end
end

function route.addPlanet(id)
  if route.currentList == nil then
    route.currentList = {}
    route.itemList = {}
  end
  table.insert(route.currentList, id)
  route.selectedPlanet = 1
  route.selectedItem = 1
  route.refreshLists = true
  dbg.add(id.." added to list.", "ROUTE")
end

function route.textInput(t)
  if route.ui:textInput(t) then return end
end

function route.keyPressed(key, code, isRepeat)
  if route.ui:keyPressed(key, code, isRepeat) then return end
  if not isRepeat then
    if key == "escape" then
      engine.state.set("game")
    end
  end
end

function route.mousePressed(x, y, b)
  if route.ui:mousePressed(x, y, b) then return end
end

function route.mouseReleased(x, y, b)
  if route.ui:mouseReleased(x, y, b) then return end
  local target = game.world:checkClick(x, y, b)
  if target ~= nil then
    if b == 1 then
      route.addPlanet(target)
      dbg.add("Planet "..target.." added to route.", "ROUTE")
    elseif b == 2 then
      local key = "planet_"..target
      if engine.state[engine.currentState].ui:windowExists(key) then
        engine.state[engine.currentState].ui:selectWindow(key)
      else
        engine.state[engine.currentState].ui:add(key, engine.window.planet_window:new(), target)
      end
    end
  end
end

function route.wheelMoved(...)
  if route.ui:wheelMoved(...) then return end
  game.world:wheelMoved(...)
end

function route.update(...)
  route.ui:updateMaster(...)
  game.world:update(...)
end

function route.drawCurrentRoute()
  if route.currentList ~= nil then
    local verts = {}
    for i = 1, #route.currentList do
      if route.currentList[i + 1] ~= nil then
        local transform = game.world:getObjectData(route.currentList[i], "transform")
        local x, y = game.world.cameras.main:getScreenPos(transform.x, transform.y)
        local target = game.world:getObjectData(route.currentList[i + 1], "transform")
        local tx, ty = game.world.cameras.main:getScreenPos(target.x, target.y)
        table.insert(verts, {x, y, tx, ty})
      end
    end
    if verts[1] ~= nil then
      love.graphics.setCanvas()
      love.graphics.setColor(0,255,0,255)
      for i = 1, #verts do
        love.graphics.line(verts[i][1], verts[i][2], verts[i][3], verts[i][4])
      end
      love.graphics.line(verts[1][1], verts[1][2], verts[#verts][3], verts[#verts][4])
    end
  end
end

function route.draw()
  game.world:draw()
  route.drawCurrentRoute()
  route.ui:draw()
end

return route