local TILE = "standard_spreadsheet_tile"

local header = engine.ui.button:extend()
      function header:setup()
        self:setTileDimensions(32, 16, 96, 16)
      end

local checkBox = engine.ui.check_box:extend()
      function checkBox:findIndex()
        for i = 1, #self.box.itemList do
          if self.box.itemList[i] == self.label then
            return i
          end
        end
      end
      function checkBox:onCheck()
        table.insert(self.box.itemList, self.label)
      end
      function checkBox:onUncheck()
        local index = self:findIndex()
        table.remove(self.box.itemList, index)
      end

local itemBtn = engine.ui.button:extend()
      function itemBtn:onClick(x, y, b)
        
      end
      function itemBtn:setup()
        self:setTileDimensions(32, 16, 96, 16)
        self.align = "left"
      end

local volBtn = engine.ui.button:extend()
      function volBtn:setup()
        self:setTileDimensions(32, 16, 96, 16)
      end

local itemBox = engine.ui.container:extend()
      function itemBox:fillList()
        local iTab = {}
        local tCol = engine.system.vars.user.uiText
        local btnWid = engine.items.getLongestString()
        for k, v in pairs(engine.items) do
          if type(engine.items[k]) == "table" then
            table.insert(iTab, k)
          end
        end
        table.sort(iTab)
        for i = 1, #iTab do
          local smallBtn = 40
          local x = 24
          local y = self:getLowest()
          if i == 1 then
            y = 8
          end
          self:add(iTab[i].."_header", header:new(iTab[i], x, y, btnWid, 16, TILE.."_hover", TILE.."_hover", tCol, true))
          x = self:stackRight(iTab[i].."_header", 0)
          self:add(iTab[i].."_volume", header:new("Vol", x, y, smallBtn, 16, TILE.."_hover", TILE.."_hover", tCol, true))
          local itemList = engine.items.getSortedTierList(iTab[i])
          for j = 1, #itemList do
            local item = engine.items.getData(itemList[j])
            x = 6
            y = y + 16
            self:add(itemList[j], checkBox:new(itemList[j], x, y, 16, 16, "check_box_off", "check_box_on", {0,0,0,0}, false))
            x = self:stackRight(itemList[j], 2)
            self:add(itemList[j].."_label", itemBtn:new(item.label, x, y, btnWid, 16, TILE, TILE.."_hover", tCol, true))
            x = self:stackRight(itemList[j].."_label", 0)
            self:add(itemList[j].."_volume", volBtn:new(item.volume, x, y, smallBtn, 16, TILE, TILE.."_hover", tCol, true)) 
          end
        end
      end
      function itemBox:setup()
        self.itemList = {}
        self:fillList()
      end

local addItems = engine.ui.button:extend()
      function addItems:onClick(x, y, b)
        local itemList = self.box.objects.items.itemList
        if engine.state.route_planner.itemList == nil then return end
        local planetIndex = engine.state.route_planner.selectedPlanet
        local planetID = engine.state.route_planner.currentList[planetIndex]
        local system = game.world.objects[planetID].localStar
        if engine.state.route_planner.itemList[planetIndex] == nil then
          engine.state.route_planner.itemList[planetIndex] = {}
        end
        for i = 1, #itemList do
          local itemData = engine.items.getData(itemList[i])
          local avg = engine.commands.getAveragePrice(system, itemList[i])
          local t = {
                      item = itemList[i],
                      maxBuyPrice = math.ceil(avg),
                      minSellPrice = math.ceil(avg + (avg * .1)),
                      maxBuyAmount = 100,
                      maxSellAmount = 100,
                    }
          table.insert(engine.state.route_planner.itemList[planetIndex], t)
        end
        engine.state.route_planner.refreshLists = true
      end

local goods = engine.ui.container:extend()

function goods:setup(x, y)
  local tCol = engine.system.vars.user.uiText
  local trans = {0,0,0,0}
  self.label = "Trade Goods"
  self:setPosition(x, y)
  self:setSize(300, 256)
  self:setBackground("standard_container")
  self:add("header", engine.ui.label:new(self.label, 0, 5, self.w, 16, trans, trans, tCol, false))
  self:add("items", itemBox:new("items", 4, self:stackUnder("header", 2), self.w - 8, 174, "standard_container", "standard_container", tCol, true))
  self:add("add_button", addItems:new("Add Selected", 4, self:stackUnder("items", 2), self.w - 8, 32, "standard_button", "standard_button_hover", tCol, true))
end

return goods