local nameWidth = 116
local inputWidth = 38

local function newNameLabel(l, x, y, w, h)
  local tCol = engine.system.vars.user.uiText
  local label = engine.ui.button:extend()
        label:setTileDimensions(32, 16, 96, 16)
        function label:setup()
          if self.box.hasScrollBar then
            self:setSize(self.w - 32, self.h)
          end
          self.index = index
          if not h then
            self.align = "left"
          end
          self.highlight = h
        end
        function label:update(dt)
          if self.highlight then
            self.isHovered = true
          end
        end
        function label:onClick(x, y, b)
          
        end
  return label:new(l, x, y, w, 16, "standard_spreadsheet_tile", "standard_spreadsheet_tile_hover", tCol, true)
end

local function newRemoveButton(y, index)
  local rb = engine.ui.button:extend()
        function rb:setup()
          self.isPicture = true
        end
        function rb:onClick(x, y, b)
          local planetIndex = engine.state.route_planner.selectedPlanet
          table.remove(engine.state.route_planner.itemList[planetIndex], index)
          engine.state.route_planner.selectedItem = math.max(engine.state.route_planner.selectedItem - 1, 1)
          engine.state.route_planner.refreshLists = true
        end
  return rb:new(index, 5, y, 16, 16, "remove_button", "remove_button_hover", {0,0,0,0}, false)
end

local function newNumberInput(item, x, y, key)
  local tCol = engine.system.vars.user.uiText
  local planetIndex = engine.state.route_planner.selectedPlanet
  local input = engine.ui.number_input:extend()
        input:setTileDimensions(32, 16, 96, 16)
        function input:getIndex()
          for i = 1, #engine.state.route_planner.itemList[planetIndex] do
            if engine.state.route_planner.itemList[planetIndex][i].item == self.item then
              return i
            end
          end
        end
        function input:setup()
          self.align = "right"
          self.item = item
          local index = self:getIndex()
          self.text = engine.state.route_planner.itemList[planetIndex][index][key]
        end
        function input:update(dt)
          local planetIndex = engine.state.route_planner.selectedPlanet
          if engine.state.route_planner.itemList == nil then return end
          if engine.state.route_planner.itemList[planetIndex] == nil then return end
          local index = self:getIndex()
          if engine.state.route_planner.itemList[planetIndex][index] == nil then return end
          engine.state.route_planner.itemList[planetIndex][index][key] = tonumber(self.text)
        end
  return input:new("", x, y, inputWidth, 16, "standard_spreadsheet_tile", "standard_spreadsheet_tile_hover", tCol, true)
end

local box = engine.ui.container:extend()

function box:setup(y)
  local trans = {0,0,0,0}
  local tCol = engine.system.vars.user.uiText
  self.label = "Stops"
  engine.state.route_planner.refreshLists = true
  self:setSize(300, 170)
  self:setPosition(5, y)
  self:setBackground("standard_container")
end

function box:refreshList()
  self.objects = nil
  self.drawOrder = nil
  if engine.state.route_planner.itemList == nil then return end
  local planetIndex = engine.state.route_planner.selectedPlanet
  local itemList = engine.state.route_planner.itemList[planetIndex]
  if itemList == nil then return end
  local x, y = 5, 5
  self:add("item_label", newNameLabel("Item", x + 18, y, nameWidth, true))
  self:add("buy_label", newNameLabel("Buy", self:stackRight("item_label", 0), y, inputWidth, true))
  self:add("buy_amount_label", newNameLabel("Amt", self:stackRight("buy_label", 0), y, inputWidth, true))
  self:add("sell_label", newNameLabel("Sell", self:stackRight("buy_amount_label", 0), y, inputWidth, true))
  self:add("sell_amount_label", newNameLabel("Amt", self:stackRight("sell_label", 0), y, inputWidth, true))
  y = self:stackUnder("item_label", 0)
  for i = 1, #itemList do
    self:add(itemList[i].item.."_remove", newRemoveButton(y, i))
    x = self:stackRight(itemList[i].item.."_remove", 2)
    self:add(itemList[i].item.."_label", newNameLabel(itemList[i].item, x, y, nameWidth))
    x = self:stackRight(itemList[i].item.."_label", 0)
    self:add(itemList[i].item.."_max_buy_price", newNumberInput(itemList[i].item, x, y, "maxBuyPrice"))
    x = self:stackRight(itemList[i].item.."_max_buy_price", 0)
    self:add(itemList[i].item.."_max_buy_amt", newNumberInput(itemList[i].item, x, y, "maxBuyAmount"))
    x = self:stackRight(itemList[i].item.."_max_buy_amt", 0)
    self:add(itemList[i].item.."_min_sell_price", newNumberInput(itemList[i].item, x, y, "minSellPrice"))
    x = self:stackRight(itemList[i].item.."_min_sell_price", 0)
    self:add(itemList[i].item.."_max_sell_amt", newNumberInput(itemList[i].item, x, y, "maxSellAmount"))
    y = y + 16
  end
end

function box:update(dt)
  if engine.state.route_planner.refreshLists then
    self:refreshList()
  end
end

return box