local WIDTH = 300

local function newRemoveButton(y, index)
  local rb = engine.ui.button:extend()
        function rb:setup()
          self.isPicture = true
        end
        function rb:onClick(x, y, b)
          table.remove(engine.state.route_planner.currentList, index)
          table.remove(engine.state.route_planner.itemList, index)
          engine.state.route_planner.selectedPlanet = math.max(engine.state.route_planner.selectedPlanet - 1, 1)
          engine.state.route_planner.refreshLists = true
        end
  return rb:new(index, 5, y, 16, 16, "remove_button", "remove_button_hover", {0,0,0,0}, false)
end

local function newNameLabel(l, x, y, index)
  local tCol = engine.system.vars.user.uiText
  local label = engine.ui.button:extend()
        label:setTileDimensions(32, 16, 96, 16)
        function label:setup()
          if self.box.hasScrollBar then
            self:setSize(self.w - 32, self.h)
          end
          self.index = index
          self.align = "left"
        end
        function label:update(dt)
          if engine.state.route_planner.selectedPlanet == self.index then
            self.isHovered = true
          end
        end
        function label:onClick(x, y, b)
          engine.state.route_planner.selectedPlanet = self.index
          engine.state.route_planner.refreshLists = true
        end
        local w = (WIDTH - 10) - (16 * 4)
  return label:new(l, x, y, w, 16, "standard_spreadsheet_tile", "standard_spreadsheet_tile_hover", tCol, true)
end

local function newUpButton(x, y, index)
  local tCol = engine.system.vars.user.uiText
  local up = engine.ui.button:extend()
        function up:onClick(x, y, b)
          if index > 1 then
            local value = engine.state.route_planner.currentList[index]
            table.remove(engine.state.route_planner.currentList, index)
            table.insert(engine.state.route_planner.currentList, index - 1, value)
            engine.state.route_planner.refreshLists = true
          end
        end
  return up:new("", x, y, 16, 16, "move_button_up", "move_button_up_hover", tCol, false)
end

local function newDownButton(x, y, index)
  local tCol = engine.system.vars.user.uiText
  local down = engine.ui.button:extend()
        function down:onClick(x, y, b)
          local planets = engine.state.route_planner.currentList
          if index < #planets then
            local value = engine.state.route_planner.currentList[index]
            table.remove(engine.state.route_planner.currentList, index)
            table.insert(engine.state.route_planner.currentList, index + 1, value)
            engine.state.route_planner.refreshLists = true
          end
        end
  return down:new("", x, y, 16, 16, "move_button_down", "move_button_down_hover", tCol, false)
end

local routeList = engine.ui.container:extend()
      function routeList:refreshList()
        local tCol = engine.system.vars.user.uiText
        local planets = engine.state.route_planner.currentList
        self.objects = nil
        self.drawOrder = nil
        if planets == nil then return end
        if #planets * 16 > self.h then
          self.hasScrollBar = true
        else
          self.hasScrollBar = false
        end
        local x = 5
        local y = 5
        if planets[1] ~= nil then
          for i = 1, #planets do
            local id = planets[i]
            local label = game.world.objects[id].name
            self:add(i.."_remove", newRemoveButton(y, i))
            x = self:stackRight(i.."_remove", 2)
            self:add(i.."_label", newNameLabel(label, x, y, i))
            x = self:stackRight(i.."_label", 2)
            self:add(i.."_move_up", newUpButton(x, y, i))
            x = self:stackRight(i.."_move_up", 2)
            self:add(i.."_move_down", newDownButton(x, y, i))
            y = y + 16
          end
        end
        self.scrollY = 0
        engine.state.route_planner.refreshLists = false
      end
      function routeList:update(dt)
        if engine.state.route_planner.refreshLists then
          self:refreshList()
        end
      end

local clearBtn = engine.ui.button:extend()
      function clearBtn:onClick(x, y, b)
        engine.state.route_planner.currentList = nil
        engine.state.route_planner.refreshLists = true
      end

local box = engine.ui.container:extend()

function box:setup()
  local trans = {0,0,0,0}
  local tCol = engine.system.vars.user.uiText
  self.label = "Stops"
  engine.state.route_planner.refreshLists = true
  self:setSize(WIDTH, 232)
  self:setPosition(5, 30)
  self:setBackground("standard_container")
  self:add("header", engine.ui.label:new(self.label, 5, 5, self.w - 10, 16, trans, trans, tCol, false))
  self:add("list", routeList:new(self.label, 5, self:stackUnder("header", 2), self.w - 10, 170, "standard_container", "standard_container_hover", tCol, true))
  self:add("clear", clearBtn:new("Clear", 5, self:stackUnder("list", 2), self.w - 10, 32, "standard_button", "standard_button_hover", tCol, true))
end

return box