local sound = {}
local notes = {
    c = 16.35,
    cs = 17.32,
    db = 17.32,
    d = 18.35,
    ds = 19.45,
    eb = 19.45,
    e = 20.6,
    f = 21.89,
    fs = 23.12,
    gb = 23.12,
    g = 24.5,
    gs = 25.96,
    ab = 25.96,
    a = 27.50,
    as = 29.14,
    bb = 29.14,
    b = 30.87,
  }

local function Oscillator(rate, freq)
  local phase = 0
  return function()
    phase = phase + 2*math.pi/rate            
    if phase == 2*math.pi then
      phase = phase - 2*math.pi
    end
    return math.sin(freq * phase)
  end
end

local function Sawtooth(rate, freq, length)
  local phase = 0
  return function()
    phase = phase + 2*math.pi/rate            
    if phase == 2*math.pi then
      phase = phase - 2*math.pi
    end
    return math.tan(freq * phase)
  end
end

function sound.startup()
  local len = 5
  local rate = 44100
  local bits = 16
  local channel = 1
  local amplitude = .1
  local soundData = {
        data = love.sound.newSoundData(len*rate,rate,bits,channel),
        position = 0,
        }
        function soundData.newNote(name, oct, length, instrument)
          local freq = notes[name] * (2 ^ oct)
          local osc = instrument(rate, freq, length)
          for i = soundData.position, soundData.position + (length * rate) - 1 do
            sample = soundData.data:getSample(i) + (osc() * amplitude)
            soundData.data:setSample(i, sample)
          end
          soundData.position = soundData.position + (length * rate) - 1
        end
  local inst = Oscillator
  soundData.newNote("c", 3, 1, inst)
  soundData.newNote("d", 3, 1, inst)
  --soundData.position = 0
  soundData.newNote("e", 3, 1, inst)
  --soundData.position = 0
  soundData.newNote("f", 3, 1, inst)
  soundData.position = 0
  --soundData.newNote("c", 6, 5, Sawtooth)
  source = love.audio.newSource(soundData.data)
  love.audio.play(source)
end

return sound