local edit = {
  menuLabel = "Sprite Editor",
  init = false,
  canvasSizeX = 32,
  canvasSizeY = 32,
  scale = 1,
  currentLayer = 1,
  boxFillType = "line",
  dataLookup = {8, 16, 32, 64, 96}
  }

local function backgroundCanvas()
  local canvas = love.graphics.newCanvas(edit.canvasSizeX, edit.canvasSizeY)
  local lastCanvas = love.graphics.getCanvas()
  local flick = true
  local grey = engine.ui.colour.grey
  local darkGrey = engine.ui.colour.shade(grey, .8)
  love.graphics.setCanvas(canvas)
  for x = 0, edit.canvasSizeX do
    for y = 0, edit.canvasSizeY do
      if flick then
        engine.ui.colour.set(grey)
      else
        engine.ui.colour.set(darkGrey)
      end
      love.graphics.points(x, y)
      flick = not flick
    end
  end
  love.graphics.setCanvas(lastCanvas)
  return canvas
end

function edit.newCanvas()
  local x = (engine.screenWidth - (edit.canvasSizeX * edit.scale)) / 2
  local y = (engine.screenHeight - (edit.canvasSizeY * edit.scale)) / 2
  edit.backgroundCanvas = backgroundCanvas()
  edit.toolCanvas = love.graphics.newCanvas(edit.canvasSizeX, edit.canvasSizeY)
  edit.toolCanvas:setFilter("nearest", "nearest")
  edit.canvas = {
    engine.ui.image:new("layer_1", x, y, edit.canvasSizeX, edit.canvasSizeY, true),
  }
end

function edit.newLayer(pos)
  if pos == nil then pos = #edit.canvas + 1 end
  local x = (engine.screenWidth - (edit.canvasSizeX * edit.scale)) / 2
  local y = (engine.screenHeight - (edit.canvasSizeY * edit.scale)) / 2
  local layer = engine.ui.image:new("layer_"..pos - 1, x, y, edit.canvasSizeX, edit.canvasSizeY)
  table.insert(edit.canvas, pos, layer)
end

function edit.saveImage()
  love.filesystem.createDirectory("/Mods/"..engine.system.vars.user.modName.."/assets")
  local folder = engine.system.vars.user.modName
  local filename = engine.system.vars.user.currentFileName
  local path = "/Mods/"..folder.."/assets/"..filename..".png"
  local output = love.graphics.newCanvas(edit.canvasSizeX, edit.canvasSizeY)
  local lastCanvas = love.graphics.getCanvas()
  love.graphics.setCanvas(output)
  for i = 1, #edit.canvas do
    love.graphics.draw(edit.canvas[i].canvas, 0, 0)
  end
  output:newImageData():encode("png", path)
  love.graphics.setCanvas(lastCanvas)
  engine.assets[filename] = love.graphics.newImage(path)
  dbg.add("Image saved to "..path, "SAVE")
end

function edit.findSliderValue(x)
  for i = 1, #edit.dataLookup do
    if edit.dataLookup[i] == x then return i end
  end
end

function edit.loadImage(filename)
  dbg.add(filename, "LOAD IMAGE")
  if engine.assets[filename] == nil then return end
  local w = engine.assets[filename]:getWidth()
  local h = engine.assets[filename]:getHeight()
  local xVal = edit.findSliderValue(w)
  local yVal = edit.findSliderValue(h)
  edit.manager.objects.tools.objects.width_slider:setValue(xVal)
  edit.manager.objects.tools.objects.height_slider:setValue(yVal)
  edit.manager.objects.tools.objects.name_box.text = filename
  edit.manager.objects.tools.objects.name_box.cursor.x = #filename + 1
  edit.manager.objects.tools.objects.name_box.isDirty = true
  local imgData = engine.assets[filename]:getData()
  for x = 0, w - 1 do
    for y = 0, h - 1 do
      local col = {imgData:getPixel(x, y)}
      edit.canvas[edit.currentLayer]:setPixel(x, y, col)
    end
  end
end

function edit.startup()
  love.graphics.setDefaultFilter("nearest", "nearest", 0)
  edit.manager = engine.ui.window_manager:new()
  --edit.manager:add("filebar", engine.window.menu_bar:new())
  edit.manager:add("palette", engine.window.palette:new())
  edit.manager:add("tools", engine.window.draw_toolbar:new())
  edit.manager:add("open", engine.window.open_asset:new())
  edit.newCanvas()
end

function edit.keyPressed(key, code, isRepeat)
  dbg.add(key.." pressed.", "INPUT")
  if edit.manager:keyPressed(key, code, isRepeat) then return end
  if key == "escape" then
    if edit.toolDraw then
      edit.vertices = nil
      edit.clearToolCanvas()
      edit.toolDraw = false
    else
      engine.state.set("game_menu")
    end
  end
end

function edit.textInput(t)
  if edit.manager:textInput(t) then return end
end

function edit.getCanvasPixel(x, y)
  local xMin = (engine.screenWidth - (edit.canvasSizeX * edit.scale)) / 2
  local yMin = (engine.screenHeight - (edit.canvasSizeY * edit.scale)) / 2
  local pWid = (edit.canvasSizeX * edit.scale) / edit.canvasSizeX
  local pHig = (edit.canvasSizeY * edit.scale) / edit.canvasSizeY
  local px = math.ceil((x - xMin) / pWid) - 1
  local py = math.floor((y - yMin) / pHig)
  if px >= 0 and px < edit.canvasSizeX then
    if py >= 0 and py < edit.canvasSizeY then
      return px, py
    end
  end
  return false
end

function edit.pencilClick(x, y, b)
  local px, py = edit.getCanvasPixel(x, y)
  if b == 0 then return end
  if px ~= false then
    local col = {}
    if b == 1 then
      col = edit.manager.objects.palette.selectedCol
    elseif b == 2 then
      col = edit.manager.objects.palette.secondaryCol
    end
    edit.canvas[edit.currentLayer]:setPixel(px, py, col)
    edit.toolDraw = true
  end
end

function edit.eraserClick(x, y, b)
  local px, py = edit.getCanvasPixel(x, y)
  local col = {0,0,0,0}
  if px ~= false and b == 1 then
    edit.canvas[edit.currentLayer]:setPixel(px, py, col)
    edit.toolDraw = true
  end
end

function edit.brushClick(x, y, b)
  
end

function edit.clearToolCanvas()
  local lastCanvas = love.graphics.getCanvas()
  love.graphics.setCanvas(edit.toolCanvas)
  love.graphics.clear(0,0,0,0)
  love.graphics.setCanvas(lastCanvas)
end

function edit.drawShape(shape, canvas, col, clear, verts)
  local lastCanvas = love.graphics.getCanvas()
  love.graphics.setCanvas(canvas)
  if clear then
    love.graphics.clear(0,0,0,0)
  end
  love.graphics.setLineStyle("rough")
  love.graphics.setColor(col)
  if shape == "line" then
    love.graphics.line(verts[1], verts[2], verts[3], verts[4])
  elseif shape == "box" then
    if edit.boxFillType == "line" then
      love.graphics.rectangle("line", verts[1], verts[2], verts[3], verts[4])
    else
      love.graphics.rectangle("fill", verts[1], verts[2], verts[3], verts[4])
    end
  elseif shape == "circle" then
    local w = verts[3] / 2
    local h = verts[4] / 2
    local px = math.ceil(verts[1] + w)
    local py = math.ceil(verts[2] + h)
    love.graphics.ellipse(edit.boxFillType, px, py, w, h)
  end
  love.graphics.setCanvas(lastCanvas)
  edit.canvas[edit.currentLayer]:updatePixelData()
end

function edit.lineClick(x, y, b, hover)
  local px, py = edit.getCanvasPixel(x, y)
  local col = engine.ui.colour.black
  if px == false then
    return
  else
    if hover then
      if edit.vertices ~= nil then
        local tempVert = {
          [1] = edit.vertices[1],
          [2] = edit.vertices[2],
          [3] = px + 1,
          [4] = py + 1,
        }
        if px < tempVert[1] then
          tempVert[1] = tempVert[1] + 1
          tempVert[3] = tempVert[3] - 1
        end
        if py < tempVert[2] then
          tempVert[2] = tempVert[2] + 1
          tempVert[4] = tempVert[4] - 1
        end
        edit.drawShape("line", edit.toolCanvas, col, true, tempVert)
        tempVert = nil
      end
      return
    else
      if b == 1 then col = edit.manager.objects.palette.selectedCol end
      if b == 2 then col = edit.manager.objects.palette.secondaryCol end
      if edit.vertices == nil then
        edit.vertices = {px, py}
        engine.ui.colour.set(col)
        edit.toolDraw = true
      else
        edit.clearToolCanvas()
        edit.vertices[3] = px + 1
        edit.vertices[4] = py + 1
        if px < edit.vertices[1] then
          edit.vertices[1] = edit.vertices[1] + 1
          edit.vertices[3] = edit.vertices[3] - 1
        end
        if py < edit.vertices[2] then
          edit.vertices[2] = edit.vertices[2] + 1
          edit.vertices[4] = edit.vertices[4] - 1
        end
        edit.drawShape("line", edit.canvas[edit.currentLayer].canvas, col, false, edit.vertices)
        edit.canvas[edit.currentLayer].imageData = edit.canvas[edit.currentLayer].canvas:newImageData()
        edit.vertices = nil
        edit.toolDraw = false
      end
    end
  end
end

function edit.boxClick(x, y, b, hover, col)
  local px, py = edit.getCanvasPixel(x, y)
  if px == false then
    return
  else
    if hover then
      if edit.vertices ~= nil then
        local tempVert = edit.vertices
        tempVert[3] = (px + 1) - tempVert[1]
        tempVert[4] = py - tempVert[2]
        edit.drawShape("box", edit.toolCanvas, col, true, tempVert)
      end
      return
    else
      if edit.vertices == nil then
        edit.vertices = {px + 1, py}
        engine.ui.colour.set(col)
        edit.toolDraw = true
      else
        edit.clearToolCanvas()
        edit.vertices[3] = (px + 1) - edit.vertices[1]
        edit.vertices[4] = py - edit.vertices[2]
        edit.drawShape("box", edit.canvas[edit.currentLayer].canvas, col, false, edit.vertices)
        edit.canvas[edit.currentLayer].imageData = edit.canvas[edit.currentLayer].canvas:newImageData()
        edit.vertices = nil
        edit.toolDraw = false
      end
    end
  end
end

function edit.circleClick(x, y, b, hover, col)
  local px, py = edit.getCanvasPixel(x, y)
  if px == false then
    return
  else
    if hover then
      if edit.vertices ~= nil then
        local tempVert = edit.vertices
        tempVert[3] = px - tempVert[1]
        tempVert[4] = py - tempVert[2]
        edit.drawShape("circle", edit.toolCanvas, col, true, tempVert)
      end
      return
    else
      if edit.vertices == nil then
        edit.vertices = {px, py}
        engine.ui.colour.set(col)
        edit.toolDraw = true
      else
        edit.clearToolCanvas()
        edit.vertices[3] = px - edit.vertices[1]
        edit.vertices[4] = py - edit.vertices[2]
        edit.drawShape("circle", edit.canvas[edit.currentLayer].canvas, col, false, edit.vertices)
        edit.canvas[edit.currentLayer].imageData = edit.canvas[edit.currentLayer].canvas:newImageData()
        edit.vertices = nil
        edit.toolDraw = false
      end
    end
  end
end

function edit.checkSameColour(x, y, col)
  --Check each part of the colour,
  for i = 1, 4 do
    if col[i] ~= edit.canvas[edit.currentLayer].currentData[x + edit.canvasSizeX * y][i] then
      --If any part is not equal to the given colour then return false.
      return false
    end
  end
  --Otherwise return true.
  return true
end

function edit.seekEdge(x, y, xVel, yVel, col)
  --Increment the values x and y by the relevent velocity
  x = x + xVel
  y = y + yVel
  
  --If the pixel coordinate is within the image then,
  if x >= 0 and x <= edit.canvasSizeX - 1 and y >= 0 and y <= edit.canvasSizeY - 1 then
    
    --if the pixel is NOT the same as the target colour OR black then
    if edit.checkSameColour(x, y, col) == false and edit.checkSameColour(x, y, {0, 0, 0, 255}) == false then
      
      --set the pixel to the target colour and check the three remaining directions, not back the way we just came.
      edit.canvas[edit.currentLayer]:setPixel(x, y, col)
      if xVel ~= 1    and x > 0                    then edit.seekEdge(x, y, -1, 0, col, tab) end
      if yVel ~= -1   and y < edit.canvasSizeY - 1 then edit.seekEdge(x, y, 0, 1, col, tab)  end
      if xVel ~= -1   and x < edit.canvasSizeX - 1 then edit.seekEdge(x, y, 1, 0, col, tab)  end
      if yVel ~= 1    and y > 0                    then edit.seekEdge(x, y, 0, -1, col, tab) end
    end
  end
end

function edit.fillClick(x, y, b)
  local palette = edit.manager.objects.palette
  local col = {0,0,0,0}
  local px, py = edit.getCanvasPixel(x, y)
  if not px then return end
  if b == 1 then col = palette.selectedCol end
  if b == 2 then col = palette.secondaryCol end
  dbg.add(px..", "..py.." {"..col[1]..", "..col[2]..", "..col[3]..", "..col[4].."}.", "FILL CLICK")
  edit.seekEdge(px, py, 0, 0, col)
end

function edit.resize(w, h)
  
end

function edit.mousePressed(x, y, b, bool)
  if b ~= 0 then
    if edit.manager:mousePressed(x, y, b) then return end
  end
  local col = engine.ui.colour.black
  if b == 1 then col = edit.manager.objects.palette.selectedCol end
  if b == 2 then col = edit.manager.objects.palette.secondaryCol end
  if edit.manager.objects.tools.currentTool == "pencil" then
    if b ~= 0 then
      edit.pencilClick(x, y, b, col)
      edit.canvas[edit.currentLayer]:drawData()
    end
  elseif edit.manager.objects.tools.currentTool == "erase" then
    if b ~= 0 then
      edit.eraserClick(x, y, b)
      edit.canvas[edit.currentLayer]:drawData()
    end
  elseif edit.manager.objects.tools.currentTool == "line" then
    edit.lineClick(x, y, b, bool, col)
  elseif edit.manager.objects.tools.currentTool == "box" then
    edit.boxClick(x, y, b, bool, col)
  elseif edit.manager.objects.tools.currentTool == "circle" then
    edit.circleClick(x, y, b, bool, col)
  elseif edit.manager.objects.tools.currentTool == "fill" then
    edit.toolDraw = false
    edit.fillClick(x, y, b)
  end
end

function edit.mouseReleased(x, y, b)
  if edit.manager:mouseReleased(x, y, b) then return end
end

function edit.wheelMoved(x, y, s)
  if edit.manager:wheelMoved(x, y, s) then
    return
  end
  edit.scale = edit.scale + (s * .5)
  if edit.scale < 1 then edit.scale = 1 end
end

function edit.update(dt)
  edit.manager:updateMaster(dt)
  if edit.toolDraw then
    edit.mousePressed(engine.mousex, engine.mousey, engine.mouseButton, true)
  end
  edit.canvas[edit.currentLayer]:draw()
end

function edit.draw()
  local x = ((engine.screenWidth - (edit.canvasSizeX * edit.scale)) / 2)
  local y = ((engine.screenHeight - (edit.canvasSizeY * edit.scale)) / 2)
  engine.ui.colour.set("white")
  love.graphics.draw(edit.backgroundCanvas, x, y, 0, edit.scale)
  for i = 1, #edit.canvas do
    love.graphics.draw(edit.canvas[i].canvas, x, y, 0, edit.scale)
  end
  engine.ui.colour.set("white")
  if edit.toolDraw then
    love.graphics.draw(edit.toolCanvas, x, y, 0, edit.scale)
  end
  edit.manager:draw()
  edit.clearToolCanvas()
end

return edit