--Colour picker Tab setup...
local swatch = engine.ui.image:extend()
      function swatch:fill()
        local col = self.win.horiSliderCol
        local col2 = self.win.vertSliderCol
        local floor = math.floor
        local w = self.w
        local h = self.h
        for x = 0, w - 1 do
          for y = h - 1, 0, -1 do
            local c = {
              math.min(floor((col[1] / w) * x) + floor((col2[1] / h) * y), 255),
              math.min(floor((col[2] / w) * x) + floor((col2[2] / h) * y), 255),
              math.min(floor((col[3] / w) * x) + floor((col2[3] / h) * y), 255),
              255,
            }
            self:setPixel(x, (h - 1) - y, c)
          end
        end
        dbg.add("Swatch updated.", "COLOUR SWATCH")
      end
      function swatch:onClick(x, y, b)
        local col = self:getPixel(x, y)
        if b == 1 then
          self.win.selectedCol = col
        elseif b == 2 then
          self.win.secondaryCol = col
        end
      end
      function swatch:update(dt)
        if self.box.swatchRender then
          self:fill()
          self.box.swatchRender = false
          return
        end
        if self.isHovered then
          self.isDirty = false
        end
      end

local vSlider = engine.ui.image:extend()
      function vSlider:blend(y, col1, col2)
        for i = 0, 31 do
          local bias = (1 / 32) * i
          local col = engine.ui.colour.blend(col1, col2, bias)
          for x = 0, self.w - 1 do
            self:setPixel(x, y + i, col)
          end
        end
      end
      function vSlider:fill(parent)
        self:blend(0,   {255,0,0,255},    {255,255,255,255})
        self:blend(32,  {255,255,0,255},  {255,0,0,255})
        self:blend(64,  {0,255,0,255},    {255,255,0,255})
        self:blend(96,  {0,255,255,255},  {0,255,0,255})
        self:blend(128, {0,0,255,255},    {0,255,255,255})
        self:blend(160, {255,0,255,255},  {0,0,255,255})
        self:blend(192, {255,0,0,255},    {255,0,255,255})
        self:blend(224, {0,0,0,255},      {255,0,0,255})
      end
      function vSlider:onClick(x, y, b)
        self.win.vertSliderCol = self:getPixel(x, y)
        self.box.swatchRender = true
      end

local hSlider = engine.ui.image:extend()
      function hSlider:blend(x, col1, col2)
        for i = 0, 31 do
          local bias = (1 / 32) * i
          local col = engine.ui.colour.blend(col1, col2, bias)
          for y = 0, self.h - 1 do
            self:setPixel(x + i, y, col)
          end
        end
      end
      function hSlider:fill()
        self:blend(0,   {255,0,0,255},    {0,0,0,255})
        self:blend(32,  {255,0,255,255},  {255,0,0,255})
        self:blend(64,  {0,0,255,255},    {255,0,255,255})
        self:blend(96,  {0,255,255,255},  {0,0,255,255})
        self:blend(128, {0,255,0,255},    {0,255,255,255})
        self:blend(160, {255,255,0,255},  {0,255,0,255})
        self:blend(192, {255,0,0,255},    {255,255,0,255})
        self:blend(224, {255,255,255,255},    {255,0,0,255})
      end
      function hSlider:onClick(x, y, b)
        self.win.horiSliderCol = self:getPixel(x, y)
        self.box.swatchRender = true
      end

local pickerBox = engine.ui.container:extend()
      function pickerBox:setup()
        self:setSize(300, 300)
        self:setBackground("standard_container")
        self:add("colour_swatch", swatch:new(self.label, 31, 10, 256, 256))
        self.swatchRender = true
        self:add("vert_swatch", vSlider:new(self.label, 11, 10, 16, 256))
        self.objects["vert_swatch"]:fill()
        self:add("hori_swatch", hSlider:new(self.label, 31, 269, 256, 16))
        self.objects["hori_swatch"]:fill()
      end
      function pickerBox:update(dt, mx, my)
        if self.isHovered then
          self.isDirty = false
        end
      end

return pickerBox