local map = {}
      map.version = 0.2

function map.getNoiseValue(map, x, y, oct)
  local freq = map.freq / oct
  local amp = map.amp / oct
  return love.math.noise(x / freq, y / freq) * amp
end

function map.toXY(w, n)
  local x = n % w
  local y = n / w
  return x - 1, y - 1
end

function map.toIndex(w, x, y)
  return x + w * y
end

function map.generate(w, h, amp, freq, oct)
  local rng = love.math.newRandomGenerator(math.random(1, 50000))
  local dump = rng:random(1, 2)
  amp = amp or rng:random(50, 300)
  freq = freq or rng:random(25, 40)
  oct = oct or rng:random(5, 10)
  local min = math.min
  local max = math.max
  local gnv = map.getNoiseValue
  local toIndex = map.toIndex
  local mapMin = love.math.noise(w / 2, h / 2)
  local mapMax = mapMin
  local m = {}
  m.w = w
  m.h = h
  m.amp = amp
  m.freq = freq
  m.oct = oct
  m.data = {}
  local c = 1
  for x = 1, w do
    for y = 1, h do
      local height = 0
      for i = 1, oct do
        height = height + gnv(m, x, y, i)
      end
      mapMin = min(mapMin, height)
      mapMax = max(mapMax, height)
      m.data[c] = {h = height, index = toIndex(w, x, y)}
      c = c + 1
    end
  end
  local mapMultiplier = 255 / (mapMax - mapMin)
  for i = 1, #m.data do
    m.data[i].h = m.data[i].h * mapMultiplier
  end
  return m
end

function map.drawCanvas(m, texture)
  local canvas = love.graphics.newCanvas(m.w, m.h)
  local getPoint = map.toXY
  love.graphics.setCanvas(canvas)
  for i = 1, #m.data do
    local x, y = getPoint(m.w, m.data[i].index)
    for j = 1, #texture do
      local c = m.data[i].h
      if c < texture[j].cutoff then
        local r = c * texture[j].col[1]
        local g = c * texture[j].col[2]
        local b = c * texture[j].col[3]
        local a = 255 * texture[j].col[4]
        love.graphics.setColor(r, g, b, a)
        break
      end
    end
    love.graphics.points(x, y)
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.setCanvas()
  return canvas
end

function map.makeShadeLayer(w, h)
  local canvas = love.graphics.newCanvas(w, h)
  local alpha = 0
  local shAng = math.random(200, 210) / 100
  love.graphics.setCanvas(canvas)
  for x = w / shAng, w do
    love.graphics.setColor(0, 0, 0, alpha)
    for y = 1, h do
      love.graphics.points(x + .5, y + .5)
    end
    alpha = math.min(alpha + 50, 240)
  end
  love.graphics.setCanvas()
  return canvas
end

map.texture = {}

map.texture.earth = {
  [1] = {cutoff = 170, col = {0, .5, .8, 1}},
  [2] = {cutoff = 172, col = {1, .8, .7, 1}},
  [3] = {cutoff = 200, col = {0, .5, 0, 1}},
  [4] = {cutoff = 220, col = {0, .3, 0, 1}},
  [5] = {cutoff = 240, col = {.5, .5, .5, 1}},
  [6] = {cutoff = 9999, col = {1, 1, 1, 1}},
}

map.texture.desert = {
  [1] = {cutoff = 130, col = {.5, .5, .5, 1}},
  [2] = {cutoff = 140, col = {.9, .7, .6, 1}},
  [3] = {cutoff = 150, col = {.8, .6, .5, 1}},
  [4] = {cutoff = 160, col = {.7, .5, .4, 1}},
  [5] = {cutoff = 200, col = {.6, .4, .3, 1}},
  [6] = {cutoff = 220, col = {0, .2, 0, 1}},
  [7] = {cutoff = 9999, col = {0, .5, .9, 1}},
  }

map.texture.magma = {
  [1] = {cutoff = 160, col = {.2, 0, 0, 1}},
  [2] = {cutoff = 170, col = {1, .3, 0, 1}},
  [3] = {cutoff = 175, col = {1, 0, 0, 1}},
  [4] = {cutoff = 180, col = {1, .5, 0, 1}},
  [5] = {cutoff = 190, col = {1, 0, 0, 1}},
  [6] = {cutoff = 200, col = {1, .3, 0, 1}},
  [7] = {cutoff = 220, col = {1, 0, 0, 1}},
  [8] = {cutoff = 9999, col = {.1, 0, 0, 1}},
  }

map.texture.moon = {
  [1] = {cutoff = 260, col = {1, 1, 1, 1}},
  }

map.clouds = {}

map.clouds.none = {
  [1] = {cutoff = 260, col = {1, 1, 1, 0}},
  }

map.clouds.white = {
  [1] = {cutoff = 145, col = {1, 1, 1, 0}},
  [2] = {cutoff = 150, col = {1, 1, 1, .45}},
  [3] = {cutoff = 155, col = {1, 1, 1, .5}},
  [4] = {cutoff = 160, col = {1, 1, 1, .55}},
  [5] = {cutoff = 165, col = {1, 1, 1, .6}},
  [6] = {cutoff = 170, col = {1, 1, 1, .65}},
  [7] = {cutoff = 175, col = {1, 1, 1, .7}},
  [8] = {cutoff = 180, col = {1, 1, 1, .75}},
  [9] = {cutoff = 190, col = {1, 1, 1, .8}},
  [10] = {cutoff = 200, col = {1, 1, 1, .85}},
  [11] = {cutoff = 210, col = {1, 1, 1, .9}},
  [12] = {cutoff = 230, col = {1, 1, 1, .95}},
  [13] = {cutoff = 260, col = {1, 1, 1, 1}},
  }

map.clouds.red = {
  [1] = {cutoff = 145, col = {1, 0, 0, 0}},
  [2] = {cutoff = 150, col = {1, 0, 0, .45}},
  [3] = {cutoff = 155, col = {1, 0, 0, .5}},
  [4] = {cutoff = 160, col = {1, 0, 0, .55}},
  [5] = {cutoff = 165, col = {1, 0, 0, .6}},
  [6] = {cutoff = 170, col = {1, 0, 0, .65}},
  [7] = {cutoff = 175, col = {1, 0, 0, .7}},
  [8] = {cutoff = 180, col = {1, 0, 0, .75}},
  [9] = {cutoff = 190, col = {1, 0, 0, .8}},
  [10] = {cutoff = 200, col = {1, 0, 0, .85}},
  [11] = {cutoff = 210, col = {1, 0, 0, .9}},
  [12] = {cutoff = 230, col = {1, 0, 0, .95}},
  [13] = {cutoff = 260, col = {1, 0, 0, 1}},
  }

return map