local player = engine.class.base:extend()

function player:init(name, cash, homeworld, aiType)
  self.name = name
  self.cash = cash
  self.home = homeworld
  self.ai = aiType
  self.currentShip = 1
  self.colour = {0,255,0,255}
  self.ships = {}
  self.buildings = {}
  self.selected = {}
  self.routes = {}
end

function player:setColour(r, g, b)
  self.colour = {r, g, b, 255}
end

--SHIP BASED FUNCTIONS--

function player:addShip(id)
  local position = #self.ships + 1 or 1
  self.ships[position] = id
  if position == 1 then
    self.currentShip = id
    if self.id == game.currentPlayer then
      CURRENT_SHIP = id
    end
  end
end

local function checkRecipe(planet, class)
  local template = engine.ships[class].new()
  local colony = game.world:getObjectData(planet, "colony")
  for i = 1, #template.recipe do
    local item = template.recipe[i].item
    local amt = template.recipe[i].amt
    if colony.inventory[item].amt < amt then
      return false
    end
  end
  for i = 1, # template.recipe do
    local item = template.recipe[i].item
    local amt = template.recipe[i].amt
    colony.inventory[item].amt = colony.inventory[item].amt - amt
  end
  return true
end

function player:buyShip(class, planet, price)
  if self.cash >= price then
    if checkRecipe(planet, class) or price == 0 then
      dbg.add(self.name.."["..self.id.."]".." buying "..class.." ship for "..price.." from "..planet, "TRADE")
      self.cash = self.cash - price
      local t = game.world:getObjectData(planet, "transform")
      local id = game.world:add(engine.ships[class].new(self.id, t.x, t.y))
      self:addShip(id)
      game.world.objects[id].target.docked = planet
      game.world.objects[id].target.id = 0
      UPDATE_PLAYER_DATA = true
      return id
    end
  end
  return 0
end

function player:selectShip(id, clear)
  if clear ~= nil then
    self.selected = nil
    self.selected = {}
  end
  table.insert(self.selected, 1, id)
end

function player:moveTo(targetId)
  local dist = game.world:getObjectDistance(CURRENT_SHIP, targetId)
  game.world.objects[CURRENT_SHIP].target.docked = 0
  game.world.objects[CURRENT_SHIP].target.id = targetId
  game.world.objects[CURRENT_SHIP].target.distance = dist
  game.world.objects[CURRENT_SHIP].target.range = dist
end

function player:assignDockOrder(targetId)
  if self.selected[1] ~= nil then
    for i = 1, #self.selected do
      local id = self.selected[i]
      local dist = game.world:getObjectDistance(id, targetId)
      game.world.objects[id].target.docked = 0
      game.world.objects[id].target.id = targetId
      game.world.objects[id].target.distance = dist
      game.world.objects[id].target.range = dist
    end
  end
end

function player:moveShipToPlanet(tShip, tPlanet)
  local dist = game.world:getObjectDistance(tShip, tPlanet)
  game.world.objects[tShip].target.docked = 0
  game.world.objects[tShip].target.id = tPlanet
  game.world.objects[tShip].target.distance = dist
  game.world.objects[tShip].target.range = dist
end

--ROUTE BASED FUNCTIONS--

function player:addRoute(key, planets, itemLists)
  if self.routes == nil then self.routes = {} end
  self.routes[key] = {planets = planets, items = itemLists,}
end

return player