local sys = {
  moonRads = {11, 17, 23, 29},
  moonRadModifier = 500,
  moonChance = 50,
  planetRads = {5, 11, 23, 41, 61, 83, 101, 113},
  planetRadModifier = 20000,
  planetChance = 66,
  landTypes = {
    "habitable",
    "magma",
    "rock",
    "gas",
    "ice",
  },
  atmoTypes = {
    "none",
    "oxygen",
    "boron",
    "hydrogen",
  },
  }

function sys.getType(pos)
  local min, max = 1, 4
  if pos < 3 then
    min = 2
  elseif pos > 5 then
    min = 3
    max = 5
  end
  local land = math.random(min, max)
  if pos == 3 then
    local int = math.random(1, 100)
    if int > 50 then
      return "earth", "oxygen"
    elseif int > 25 then
      return "desert", "oxygen"
    else
      return "rock", "oxygen"
    end
  end
  if land == 1 then land = 3 end
  return sys.landTypes[land], sys.atmoTypes[math.random(1, #sys.atmoTypes)]
end

function sys.newOrbiter(class, parentId, starId, radius, scale)
  return {
    type = class,
    localStar = starId,
    transform = engine.component.transform.new(0, 0, scale * 10),
    orbiter = engine.component.orbiter.new(parentId, radius),
  }
end

function sys.newMoon(parentId, starId, orbitPos, landType, atmoType, name)
  local texSize = engine.system.vars.planet.textureSize
  local radius = sys.moonRads[orbitPos] * sys.moonRadModifier
  local m = sys.newOrbiter("moon", parentId, starId, radius, (math.random(2, 4) * 2) / 10)
  m.planet = engine.component.planet.new(landType, atmoType)
  m.renderer = engine.component.renderer.new(landType, texSize, texSize, "fisheye", 0)
  m.resource = engine.component.resource.new(landType)
  m.localStar = starId
  if name ~= nil then
    m.name = name
  else
    m.name = game.world.objects[parentId].name.." "..orbitPos
  end
  table.insert(game.world.objects[starId].planets, game.world:add(m))
end

function sys.newPlanet(starId, orbitPos, landType, atmoType, moons, name)
  if landType == nil or atmoType == nil then
    landType, atmoType = sys.getType(orbitPos)
  end
  local texSize = engine.system.vars.planet.textureSize
  local radius = sys.planetRads[orbitPos] * sys.planetRadModifier
  local moonRad = sys.moonRadModifier
  local size = math.random(9, 12) * 2
  if landType == "gas" then
    size = size * 2
    moonRad = moonRad * 2
  end
  local p = sys.newOrbiter("planet", starId, starId, radius, math.ceil(size / 10))
  p.planet = engine.component.planet.new(landType, atmoType)
  p.renderer = engine.component.renderer.new(landType, texSize, texSize, "fisheye", 0)
  p.resource = engine.component.resource.new(landType)
  if name ~= nil then
    p.name = name
  else
    p.name = game.world.objects[starId].name.." "..orbitPos
  end
  p.localStar = starId
  local planetId = game.world:add(p)
  table.insert(game.world.objects[starId].planets, planetId)
  dbg.add("New Planet "..planetId..": "..p.planet.landType..", "..p.planet.atmoType..".", "WORLD SPAWN")
  if moons then
    for i = 1, #sys.moonRads do
      local rnd = math.random(1, 100)
      if rnd < sys.moonChance then
        landType, atmoType = sys.getType(orbitPos)
        sys.newMoon(planetId, starId, i, landType, atmoType)
        dbg.add("New Moon "..planetId..": "..landType..", "..atmoType..".", "WORLD SPAWN")
      end
    end
  end
  sys.moonChance = math.min(sys.moonChance + 20, 85)
end

function sys.newStar(x, y)
  local planetCount = 0
  local starId = game.world:add(engine.stars.white_dwarf.new(x, y, "Test "..#game.world.stars))
  if game.world.stars == nil then game.world.stars = {} end
  table.insert(game.world.stars, starId)
  game.world.objects[starId].planets = {}
  sys.moonChance = 0
  for i = 1, #sys.planetRads do
    local rnd = math.random(1, 100)
    if rnd < sys.planetChance or i == 3 then
      sys.newPlanet(starId, i, nil, nil, true)
    end
  end
end

function sys.newSolaris(x, y)
  local planetCount = 0
  local starId = game.world:add(engine.stars.white_dwarf.new(x, y, "Sol"))
  if game.world.stars == nil then game.world.stars = {} end
  table.insert(game.world.stars, starId)
  game.world.objects[starId].planets = {}
  sys.newPlanet(starId, 1, "rock", "boron", false, "Mercury")
  sys.newPlanet(starId, 2, "magma", "hydrogen", false, "Venus")
    sys.newMoon(3, starId, 2, "rock", "none")
  sys.newPlanet(starId, 3, "earth", "oxygen", false, "Earth")
    sys.newMoon(5, starId, 3, "rock", "none", "Lua")
  sys.newPlanet(starId, 4, "desert", "none", false, "Mars")
    sys.newMoon(7, starId, 2, "rock", "none")
  sys.newPlanet(starId, 5, "gas", "hydrogen", true, "Jupiter")
  sys.newPlanet(starId, 6, "gas", "boron", true, "Saturn")
  sys.newPlanet(starId, 7, "ice", "none", true, "Uranus")
end

return sys