local asset = engine.class.base:extend()
      asset.quadSizeX = 32
      asset.quadSizeY = 32
      asset.imageWidth = 96
      asset.imageHeight = 32
      asset.quads = {
        left =  love.graphics.newQuad( 0, 0, asset.quadSizeX, asset.quadSizeY, asset.imageWidth, asset.imageHeight),
        mid =   love.graphics.newQuad(32, 0, asset.quadSizeX, asset.quadSizeY, asset.imageWidth, asset.imageHeight),
        right = love.graphics.newQuad(64, 0, asset.quadSizeX, asset.quadSizeY, asset.imageWidth, asset.imageHeight),
      }

function asset:init(label, x, y, w, h, bCol, hCol, tCol, tiled)
  if tiled == nil then tiled = false end
  self.isVisible = true
  self.isHovered = false
  self.isMoveable = false
  self.isGrabbed = false
  self.isSelected = false
  self.isDirty = true
  self.drawLayer = 0
  self.isTiled = tiled
  if x ~= nil then
    self:setPosition(x, y)
  end
  if w ~= nil then
    self:setSize(w, h)
  end
  self:setBackground(bCol)
  self:setHoverColour(hCol)
  self:setTextColour(tCol)
  if label ~= nil then
    self.label = label
  else
    self.label = "UNNAMED!!!"
  end
  self:internalSetup()
end

function asset:internalSetup() end

function asset:setup() end

function asset:onOpen() end

function asset:onClose() end

function asset:setLabel(label)
  self.label = label
end

function asset:setPosition(x, y)
  local w, h = self:getSize()
  if x < 0 then x = 0 end
  if x > engine.screenWidth - w then x = engine.screenWidth - w end
  if y < 0 then y = 0 end
  if y > engine.screenHeight - h then y = engine.screenHeight - h end
  self.x = math.floor(x + .5)
  self.y = math.floor(y + .5)
end

function asset:getPosition()
  return self.x, self.y
end

function asset:setSize(w, h)
  if self.xScale == nil then self.xScale = 1 end
  if self.yScale == nil then self.yScale = 1 end
  self.w = w
  self.h = h
  if self.canvas ~= nil then
    self.canvas = love.graphics.newCanvas(w, h)
    self.isDirty = true
  end
end

function asset:getRealSize()
  return self.w, self.h
end

function asset:setScale(x, y)
  self.xScale = x
  self.yScale = y
  self.isDirty = true
end

function asset:getScaledPosition()
  if self.xScale == nil then self.xScale = 1 end
  if self.yScale == nil then self.yScale = 1 end
  return self.x * self.xScale, self.y * self.yScale
end

function asset:getSize()
  if self.w == nil then
    return 1, 1
  end
  return self.w * self.xScale, self.h * self.yScale
end

function asset:setBackground(dat)
  if type(dat) == "string" then
    self.image = dat
  else
    self.bCol = dat
  end
end

function asset:setHoverColour(dat)
  if type(dat) == "string" then
    self.hoverImage = dat
  else
    self.hCol = dat
  end
end

function asset:setTextColour(dat)
  if type(dat) == "string" then
    self.hoverImage = dat
  else
    self.tCol = dat
  end
end

function asset:setTileDimensions(sx, sy, w, h)
  self.quadSizeX = sx
  self.quadSizeY = sy
  self.quads = {
        left = love.graphics.newQuad(0, 0, sx, sy, w, h),
        mid = love.graphics.newQuad(sx, 0, sx, sy, w, h),
        right = love.graphics.newQuad(sx * 2, 0, sx, sy, w, h),
      }
end

function asset:setVisible(bool)
  self.isVisible = bool
end

function asset:toggleVisible()
  self.isVisible = not self.isVisible
end

function asset:setMoveable(bool)
  self.isMoveable = bool
end

function asset:toggleMoveable()
  self.isMoveable = not self.isMoveable
end

function asset:keyPressed(k, c, r)
  
end

function asset:checkHovered(x, y)
  local xPos, yPos = self.x, self.y
  if self.win ~= nil or self.box ~= nil then
    xPos, yPos = self:getScaledPosition()
  end
  local w, h = self:getSize()
  local wasHovered = self.isHovered
  self.isHovered = false
  if x >= xPos and x <= xPos + w then
    if y >= yPos and y <= yPos + h then
      self.isHovered = true
      if self.type ~= "container" and self.type ~= "image" then
        self.isDirty = true
      end
      return true
    end
  end
  if wasHovered and self.isHovered == false then
    self.isDirty = true
    if self.box ~= nil then
      if self.canvas == nil then
        self.box.isDirty = true
      end
    end
  end
  return false
end

function asset:mouseReleased(x, y, b)
  self.isGrabbed = false
  if self.isVisible then
    if self:checkHovered(x, y) then
      if self.box ~= nil then self.box.isDirty = true end
      if self.win ~= nil then self.win.isDirty = true end
      self:onClick(x, y, b)
      return true
    else
      if self.isSelected then
        self.isDirty = true
      end
      self.isSelected = false
    end
  end
  return false
end

function asset:mousePressed(x, y, b)
  if self.isVisible then
    self.isSelected = false
    if self:checkHovered(x, y) then
      if b == 1 then
        self.grabx = x - self.x
        self.graby = y - self.y
        self.isGrabbed = true
        self:onClickDown(x, y, b)
        return true
      end
    end
  end
  return false
end

function asset:wheelMoved(mx, my, s)
  if self:checkHovered(mx, my) then
    self:onScroll(s)
    return true
  end
  return false
end

function asset:updateMaster(dt, mx, my)
  if mx == nil then
    mx, my = engine.mousex, engine.mousey
  end
  self:checkHovered(mx, my)
  if self.isMoveable and self.isGrabbed then
    local newx = mx - self.grabx
    local newy = my - self.graby
    self:setPosition(newx, newy)
  end
  self:update(dt, mx, my)
end

function asset:tileBackground()
  local image = self.image
  local x, y = self:getPosition()
  local w, h = self:getRealSize()
  local size = self.quadSizeX
  local scaleX = (w - (2 * self.quadSizeX)) / self.quadSizeX
  local scaleY = h / self.quadSizeY
  if self.box ~= nil then
    y = y - (self.box.scrollY or 0)
  end
  love.graphics.setColor(engine.system.vars.user.uiColour)
  if self.isHovered or self.isSelected and self.hoverImage ~= nil then
    image = self.hoverImage
  end
  love.graphics.draw(engine.assets[image], self.quads.mid, x + self.quadSizeX, y, 0, scaleX, scaleY)
  love.graphics.draw(engine.assets[image], self.quads.left, x, y)
  love.graphics.draw(engine.assets[image], self.quads.right, (x + w) - self.quadSizeX, y)
end

function asset:drawBackground()
  if self.isDirty then
    local x, y = self.x, self.y
    local w, h = self:getRealSize()
    if self.box ~= nil then
      y = y - self.box.scrollY
    end
    if engine.assets[self.image] ~= nil then
      love.graphics.setColor(engine.system.vars.user.uiColour)
      if not self.isHovered then
        love.graphics.draw(engine.assets[self.image], x, y)
      else
        love.graphics.draw(engine.assets[self.hoverImage], x, y)
      end
    else
      engine.ui.colour.set(self.bCol)
      love.graphics.rectangle("fill", x, y, w, h)
    end
    if self.isHovered and self.hoverImage == nil then
      engine.ui.colour.set(engine.ui.colour.black)
      love.graphics.rectangle("line", x, y, w, h)
    end
    self.isDirty = false
  end
end

function asset:getCanvas()
  if self.canvas then
    local lastCanvas = love.graphics.getCanvas()
    love.graphics.setCanvas(self.canvas)
    return lastCanvas
  elseif self.box ~= nil then
    local lastCanvas = love.graphics.getCanvas()
    love.graphics.setCanvas(self.box.canvas)
    return lastCanvas
  elseif self.win ~= nil then
    local lastCanvas = love.graphics.getCanvas()
    love.graphics.setCanvas(self.win.canvas)
    return lastCanvas
  end
  return false
end

function asset:draw()
  if self.isVisible then
    if self.isDirty then
      local lastCanvas = self:getCanvas()
      self:drawBackground()
      love.graphics.setCanvas(lastCanvas)
      self.isDirty = false
    end
  end
end

function asset:update(dt) end

function asset:onClick(x, y, b) end

function asset:onClickDown(x, y, b) end

function asset:onScroll(y)
  if self.box ~= nil then
    if self.box.objects.scroll_bar ~= nil then
      if y > 0 then
        self.box.objects.scroll_bar:decrease()
      else
        self.box.objects.scroll_bar:increase()
      end
    end
  end
end

function asset:textInput(t) end

return asset