local button = engine.ui.asset:extend()

function button:internalSetup()
  self.type = "button"
  self.align = "center"
  self.showText = true
end

function button:drawBackground()
  local x, y = self:getPosition()
  if self.box ~= nil then
    y = y - (self.box.scrollY or 0)
  end
  if self.image ~= nil then
    if self.isPicture then
      engine.ui.colour.set("white")
    else
      engine.ui.colour.set(engine.system.vars.user.uiColour)
    end
    if self.isTiled then
      self:tileBackground()
    else
      if self.isHovered and self.hoverImage ~= nil then
        love.graphics.draw(engine.assets[self.hoverImage], x, y)
      elseif self.isSelected and self.hoverImage ~= nil then
        love.graphics.draw(engine.assets[self.hoverImage], x, y)
      else
        love.graphics.draw(engine.assets[self.image], x, y)
      end
    end
  else
    engine.ui.colour.set(self.bCol)
    love.graphics.rectangle("fill", x, y, self.w, self.h)
  end
  if self.hoverImage == self.image and self.isHovered then
    engine.ui.colour.set(self.hoverColour or "black")
    love.graphics.rectangle("line", x, y, self.w, self.h)
  end
  if self.showText then
    local str = self.label
    local w = font:getWidth(str)
    local h = font:getHeight()
    local px = x + math.floor((self.w - w) / 2)
    local py = y + math.floor((self.h - h) / 2)
    if self.align == "left" then
      px = self.x + 5
    elseif self.align == "right" then
      px = self.w - font:getWidth(str)
    end
    engine.ui.colour.set(self.tCol)
    love.graphics.print(str, px, py)
  end
end

function button:closeBtn(x, y)
  local b = button:extend()
  function b:update(dt)
    self.isDirty = true
  end
  function b:onClick(x, y, b)
    self.win.manager:closeWindow(self.win.key)
  end
  return b:new("", x, y, 32, 32, "close_button", "close_button_hover", {0,0,0,0}, false)
end

function button:stretchBtn()
  local b = button:extend()
        function b:onClickDown(x, y, b)
          self.stretch = true
          self.startWidth = self.win.w * self.win.xScale
          self.startHeight = self.win.h * self.win.yScale
          dbg.add("Stretch clicked, scale at "..self.win.xScale..", "..self.win.yScale, "STRETCH")
        end
        function b:update(dt, mx, my)
          self:setPosition(self.win.w - self.w, self.win.h - self.h)
          if engine.mouseButton == 1 then
            if self.stretch then
              local w, h = self.win.canvas:getDimensions()
              local newWid = self.win.x + (mx - self.win.x)
              local newHig = self.win.y + (my - self.win.y)
              local sx = math.round(newWid / w, 2)
              local sy = math.round(newHig / h, 2)
              dbg.add(sx..", "..sy, "STRETCH")
              self.win:setScale(sx, sy)
            end
          else
            self.stretch = false
          end
        end
  return b:new("", 1, 1, 10, 10, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, false)
end

return button