local container = engine.ui.asset:extend()
      container.quadSize = 32
      container.imageSize = 96
      container.quads = {
        top_left = love.graphics.newQuad(0, 0, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        top = love.graphics.newQuad(container.quadSize, 0, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        top_right = love.graphics.newQuad(64, 0, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        mid_left = love.graphics.newQuad(0, container.quadSize, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        mid = love.graphics.newQuad(container.quadSize, container.quadSize, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        mid_right = love.graphics.newQuad(64, container.quadSize, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        bottom_left = love.graphics.newQuad(0, 64, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        bottom = love.graphics.newQuad(container.quadSize, 64, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
        bottom_right = love.graphics.newQuad(64, 64, container.quadSize, container.quadSize, container.imageSize, container.imageSize),
      }

function container:newScrollBar()
  local scrollBar = engine.ui.slider_vert:extend()
        function scrollBar:update(dt)
          if not self.isInitialized then
            self:setLimits(0, self.box:getLowest() - self.box.h + 16, 16)
            self:setValue(0)
            self:setSize(32, self.box.h - 8)
            self.isInitialized = true
          end
        end
        function scrollBar:onValueChange()
          self.box.scrollY = self.value
          self.box.isDirty = true
        end
  return scrollBar:new("yAxis", self.w - 36, 4, 32, self.h - 8, "standard_slider", "standard_slider", engine.system.vars.user.uiText, true)
end

function container:internalSetup()
  self.type = "container"
  self.objects = {}
  self.canvas = love.graphics.newCanvas(self.w, self.h)
  self.scrollY = 0
end

function container:stackRight(label, spacing)
  local o = self.objects[label]
  return o.x + o.w + spacing
end

function container:stackUnder(label, spacing)
  local o = self.objects[label]
  return o.y + o.h + spacing
end

function container:getLowest()
  local lowest = 1
  for k, v in pairs(self.objects) do
    local check = self.objects[k].y + self.objects[k].h
    if check > lowest then
      lowest = check
    end
  end
  return lowest
end

function container:setScale(x, y)
  self.xScale = x
  self.yScale = y
  for k, v in pairs(self.objects) do
    self.objects[k]:setScale(x, y)
  end
  --self.isDirty = true
end

function container:add(key, object, ...)
  if self.objects == nil then self.objects = {} end
  if self.drawOrder == nil then self.drawOrder = {} end
  self.objects[key] = object
  self.objects[key].key = key
  if self.win == nil then
    self.objects[key].win = self
  else
    self.objects[key].win = self.win
  end
  self.objects[key].box = self
  self.objects[key].boxId = self.key
  self.objects[key]:setup(...)
  table.insert(self.drawOrder, 1, key)
  if object.y + object.h > self.h + 10 then
    self:add("scroll_bar", self:newScrollBar())
  end
end

function container:centerObject(key, axis)
  if self.objects[key] == nil then return end
  local objData = self.objects[key]
  local xoffset = (self.w - objData.w) / 2
  local yoffset = (self.h - objData.h) / 2
  if axis == nil then
    self.objects[key]:setPosition(xoffset, yoffset)
  elseif axis == "horizontal" or axis == "h" then
    self.objects[key]:setPosition(xoffset, objData.y)
  elseif axis == "vertical" or axis == "v" then
    self.objects[key]:setPosition(objData.x, yoffset)
  end
end

function container:deselectAll()
  if self.objects ~= nil then
    for k, v in pairs(self.objects) do
      self.objects[k].isSelected = false
    end
  end
end

function container:keyPressed(key, code, isRepeat)
  if self.drawOrder ~= nil then
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]]:keyPressed(key, code, isRepeat) then
        return true
      end
    end
  end
  return false
end

function container:textInput(t)
  if self.drawOrder ~= nil then
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]]:textInput(t) then
        return true
      end
    end
  end
  return false
end

function container:mousePressed(x, y, b)
  if self:checkHovered(x, y) then
    self:deselectAll()
    if self.isVisible then
      if self.drawOrder ~= nil then
        if self.type == "window" then
          x = x - self.x
          y = y - self.y
        else
          local xPos, yPos = self:getScaledPosition()
          x, y = x - xPos, y - yPos
        end
        for i = 1, #self.drawOrder do
          local scrollOffset = 0
          if self.drawOrder[i] ~= "scroll_bar" then
            scrollOffset = self.scrollY
          end
          if self.objects[self.drawOrder[i]]:mousePressed(x, y + scrollOffset, b) then
            self.isGrabbed = false
            return true
          end
        end
      end  
      self.grabx = x
      self.graby = y
      self.isGrabbed = true
      self.isDirty = true
      return true
    end
  end
  return false
end

function container:mouseReleased(x, y, b)
  self.isGrabbed = false
  if self:checkHovered(x, y) then
    if self.drawOrder ~= nil then
      if self.type == "window" then
        x = x - self.x
        y = y - self.y
      else
        local xPos, yPos = self:getScaledPosition()
        x, y = x - xPos, y - yPos
      end
      for i = 1, #self.drawOrder do
        local scrollOffset = 0
        if self.drawOrder[i] ~= "scroll_bar" then
          scrollOffset = self.scrollY
        end
        if self.objects[self.drawOrder[i]]:mouseReleased(x, y + scrollOffset, b) then
          self.isGrabbed = false
          return true
        end
      end
    end
  end
  return false
end

function container:wheelMoved(mx, my, s)
  if self:checkHovered(mx, my) then
    local x, y = 0, 0
    if self.type == "window" then
      x = mx - self.x
      y = my - self.y
    else
      local xPos, yPos = self:getScaledPosition()
      x, y = mx - xPos, my - yPos
    end
    if self.drawOrder ~= nil then
      for i = 1, #self.drawOrder do
        if self.objects[self.drawOrder[i]]:wheelMoved(x, y, s) then
          return true
        end
      end
    end
    self:onScroll(s)
    return true
  end
  return false
end

function container:updateMaster(dt, mx, my)
  if mx == nil then
    mx, my = engine.mousex, engine.mousey
  end
  self:checkHovered(mx, my)
  if self.drawOrder ~= nil then
    local scrollY = self.scrollY or 0
    local x, y = mx, my
    if self.type == "window" then
      x = x - self.x
      y = y - self.y
    else
      local xPos, yPos = self:getScaledPosition()
      x, y = x - xPos, y - yPos
    end
    for i = 1, #self.drawOrder do
      if self.drawOrder[i] ~= "scroll_bar" then
        y = y + (scrollY * self.yScale)
      end
      self.objects[self.drawOrder[i]]:updateMaster(dt, x, y)
      if self.objects[self.drawOrder[i]].y < scrollY then
        self.objects[self.drawOrder[i]].isDirty = false
        self.objects[self.drawOrder[i]].isHovered = false
      end
    end
  end
  if self.isMoveable and self.isGrabbed then
    local newx = engine.mousex - self.grabx
    local newy = engine.mousey - self.graby
    self:setPosition(newx, newy)
  end
  self:update(dt, mx, my)
  if self.isDirty then
    self:drawBackground()
    self:redrawObjects()
  elseif self.drawOrder ~= nil then
    for i = #self.drawOrder, 1, -1 do
      self.objects[self.drawOrder[i]]:draw()
    end
  end
end

function container:drawShade()
  if not self.drawShade then return end
  local topLeft = {x = 0, y = 0}
  local topRight = {x = self.w, y = 0}
  local bottomLeft = {x = 0, y = self.h}
  local bottomRight = {x = self.w, y = self.h}
  engine.ui.colour.set(engine.ui.colour.shade("black"))
  love.graphics.line(topLeft.x, topLeft.y, topRight.x, topRight.y)
  love.graphics.line(topLeft.x + 1, topLeft.y + 1, topRight.x - 1, topRight.y + 1)
  love.graphics.line(topLeft.x, topLeft.y, bottomLeft.x, bottomLeft.y)
  love.graphics.line(topLeft.x + 1, topLeft.y + 1, bottomLeft.x - 1, bottomLeft.y - 1)
  engine.ui.colour.set("white", .8)
  love.graphics.line(bottomRight.x, bottomRight.y, bottomLeft.x, bottomLeft.y)
  love.graphics.line(bottomRight.x - 1, bottomRight.y - 1, bottomLeft.x + 1, bottomLeft.y - 1)
  love.graphics.line(bottomRight.x, bottomRight.y, topRight.x, topRight.y)
  love.graphics.line(bottomRight.x - 1, bottomRight.y - 1, topRight.x - 1, topRight.y - 1)
end

function container:drawBackground()
  local lastCanvas = self:getCanvas()
  if engine.assets[self.image] ~= nil then
    love.graphics.setColor(engine.system.vars.user.uiColour)
    self:tileBackground()
  else
    engine.ui.colour.set(self.bCol)
    love.graphics.rectangle("fill", 0, 0, self.w, self.h)
    if self.isHovered then
      engine.ui.colour.set(engine.ui.colour.black)
      love.graphics.rectangle("line", 0, 0, self.w, self.h)
    end
  end
  self.isDirty = false
  love.graphics.setCanvas(lastCanvas)
end

function container:tileBackground()
  local image = self.image
  local x, y = 0, 0
  local w, h = self:getRealSize()
  local width = w - (self.quadSize * 2)
  local height = h - (self.quadSize * 2)
  local scaleX = width / self.quadSize
  local scaleY = height / self.quadSize
  local leftX = 0
  local rightX = x + (w - self.quadSize)
  local bottomY = y + (h - self.quadSize)
  love.graphics.setColor(engine.system.vars.user.uiColour)
  love.graphics.draw(engine.assets[image], self.quads.top_left, leftX, y)
  love.graphics.draw(engine.assets[image], self.quads.top, x + self.quadSize, y, 0, scaleX, 1)
  love.graphics.draw(engine.assets[image], self.quads.top_right, rightX, y)
  
  love.graphics.draw(engine.assets[image], self.quads.mid_left, x, y + self.quadSize, 0, 1, scaleY)
  love.graphics.draw(engine.assets[image], self.quads.mid, x + self.quadSize, y + self.quadSize, 0, scaleX, scaleY)
  love.graphics.draw(engine.assets[image], self.quads.mid_right, rightX, y + self.quadSize, 0, 1, scaleY)
  
  love.graphics.draw(engine.assets[image], self.quads.bottom_left, leftX, bottomY)
  love.graphics.draw(engine.assets[image], self.quads.bottom, x + self.quadSize, bottomY, 0, scaleX, 1)
  love.graphics.draw(engine.assets[image], self.quads.bottom_right, rightX, bottomY)
end

function container:redrawObjects()
  if self.drawOrder ~= nil then
    if self.scrollY == nil then self.scrollY = 0 end
    for i = #self.drawOrder, 1, -1 do
      local x, y = self.objects[self.drawOrder[i]]:getPosition()
      local w, h = self.objects[self.drawOrder[i]]:getSize()
      if (y >= self.scrollY and y + h <= self.scrollY + self.h) or self.drawOrder[i] == "scroll_bar" then
        if self.objects[self.drawOrder[i]].type ~= "image" then
          self.objects[self.drawOrder[i]].forceDraw = true
          self.objects[self.drawOrder[i]].isDirty = true
          self.objects[self.drawOrder[i]]:draw()
        end
      end
    end
  end
end

function container:draw()
  if self.isVisible then
    local lastCanvas = love.graphics.getCanvas()
    if self.box ~= nil then
      love.graphics.setCanvas(self.box.canvas)
    elseif self.win ~= nil then
      love.graphics.setCanvas(self.win.canvas)
    end
    engine.ui.colour.set("white")
    love.graphics.draw(self.canvas, self.x, self.y)
    love.graphics.setCanvas(lastCanvas)
  end
end

return container