local image = engine.ui.asset:extend()

local function generateDataTable(w, h)
  local rTab = {}
  for x = 0, w - 1 do
    for y = 0, h - 1 do
      rTab[x + w * y] = {0,0,0,0}
    end
  end
  return rTab
end

function image:init(label, x, y, w, h)
  self.isVisible = true
  self.isHovered = false
  self.isMoveable = false
  self.isGrabbed = false
  self.isSelected = true
  self.isDirty = true
  self.forceDraw = true
  self.type = "image"
  self.label = label
  self:setPosition(x, y)
  self:setSize(w, h)
  self.currentData = generateDataTable(w, h)
  self.canvas = love.graphics.newCanvas(w, h)
end

function image:setPixel(x, y, col)
  if x < 0 or x > self.w - 1 then return end
  if y < 0 or y > self.h - 1 then return end
  self.currentData[x + self.w * y] = col
  self.isDirty = true
end

function image:setCanvasPixel(x, y, col)
  engine.ui.colour.set(col)
  self.currentData[x + self.w * y] = col
  love.graphics.points(x, y)
end

function image:getPixel(x, y)
  x = x - self.x
  y = y - self.y
  return self.currentData[x + self.w * y]
end

function image:getPixelData(x, y)
  return self.imageData:getPixel(x, y)
end

function image:updatePixelData()
  self.imageData = self.canvas:newImageData()
  for x = 0, self.w - 1 do
    for y = 0, self.h - 1 do
      self.currentData[x + self.w * y] = {self:getPixelData(x, y)}
    end
  end
end

function image:drawData()
  if self.isDirty then
    local lastCanvas = self:getCanvas()
    love.graphics.setCanvas(self.canvas)
    love.graphics.clear(0,0,0,0)
    for x = 0, self.w - 1 do
      for y = 0, self.h - 1 do
        self:setCanvasPixel(x, y, self.currentData[x + self.w * y])
      end
    end
    love.graphics.setCanvas(lastCanvas)
    dbg.add(self.label.." redrawn.", "DRAW")
  end
  self.isDirty = false
  self.forceDraw = false
end

function image:draw()
  self:drawData()
  engine.ui.colour.set("white")
  local lastCanvas = love.graphics.getCanvas()
  if self.box ~= nil then
    love.graphics.setCanvas(self.box.canvas)
  elseif self.win ~= nil then
    love.graphics.setCanvas(self.win.canvas)
  end
  love.graphics.draw(self.canvas, self.x, self.y)
  love.graphics.setCanvas(lastCanvas)
end

return image