local input = engine.ui.asset:extend()

function input:internalSetup()
  self.type = "input_box"
  self.text = ""
  self.cursor = {x = 1, y = 1}
  self.align = "center"
  self.isSelected = false
  self.isVisible = true
end

function input:textInput(t)
  local x = self.cursor.x
  if self.isSelected then
    if x == 1 then
      self.text = t..self.text
    elseif x < #self.text then
      local str = self.text
      local pcVal = string.sub(str, 1, x)
      local acVal = string.sub(str, x, #str)
      self.text = pcVal..t..acVal
    else
      self.text = self.text..t
    end
    self.cursor.x = x + #t
    self.isDirty = true
    return true
  end
end

function input:onClick()
  self.isSelected = true
end

function input:keyPressed(k, code, isRepeat)
  if self.isSelected then
    if k == "backspace" then
      -- get the byte offset to the last UTF-8 character in the string.
      local byteoffset = utf8.offset(self.text, -1)

      if byteoffset then
        -- remove the last UTF-8 character.
        -- string.sub operates on bytes rather than UTF-8 characters, so we couldn't do string.sub(text, 1, -2).
        self.text = string.sub(self.text, 1, byteoffset - 1)
        self.cursor.x = math.max(self.cursor.x - 1, 1)
        self.isDirty = true
      end
      return true
    end
  end
end

function input:drawBackground()
  if self.isDirty then
    if self.image ~= nil then
      if self.isPicture then
        engine.ui.colour.set("white")
      else
        engine.ui.colour.set(engine.system.vars.user.uiColour)
      end
      if self.isTiled then
        self:tileBackground()
      else
        if self.isHovered and self.hoverImage ~= nil then
          love.graphics.draw(engine.assets[self.hoverImage], x, y)
        elseif self.isSelected and self.hoverImage ~= nil then
          love.graphics.draw(engine.assets[self.hoverImage], x, y)
        else
          love.graphics.draw(engine.assets[self.image], x, y)
        end
      end
    else
      engine.ui.colour.set(self.bCol)
      love.graphics.rectangle("fill", x, y, self.w, self.h)
    end
  end
  local scrollY = 0
  if self.box ~= nil then
    scrollY = self.box.scrollY
  end
  if self.text == nil then self.text = "" end
  local str = self.text
  local w = font:getWidth(str)
  local h = font:getHeight()
  local px = self.x + ((self.w - w) / 2)
  local py = (self.y - scrollY) + ((self.h - h) / 2)
  if self.align == "left" then
    px = self.x + 5
  elseif self.align == "right" then
    px = self.x + ((self.w - 5) - w)
  end
  if self.isSelected then
    str = str.."|"
  end
  engine.ui.colour.set(self.tCol)
  love.graphics.print(str, px, py)
  self.isDirty = false
end

return input