--Input Buttons...

local dnBtn = engine.ui.button:extend()
      function dnBtn:onClick(x, y, b)
        dbg.add(self.boxId.." Clicked")
        self.box:decrease()
      end

local upBtn = engine.ui.button:extend()
      function upBtn:onClick(x, y, b)
        dbg.add(self.boxId.." Clicked")
        self.box:increase()
      end

local slBtn = engine.ui.button:extend()
      function slBtn:setup()
        self:toggleMoveable()
      end
      function slBtn:update(dt, mx, my)
        local w, h = self.box:getSize()
        local minx = self.box.quadSize
        local maxx = w - (minx * 2)
        if self.y ~= 0 then self.y = 0 end
        if self.x < minx then self.x = minx end
        if self.x > maxx then self.x = maxx end
      end
      function slBtn:onClick(x, y, b)
        self.box:getPositionValue(self.x, self.y)
      end
      function slBtn:onScroll(s)
        dbg.add("Slider scrolled: "..s, "Slider")
        if s > 0 then
          self.box:increase()
        else
          self.box:decrease()
        end
      end

--Slider Main...

local slider = engine.ui.container:extend()
      slider.quadSizeX = 32
      slider.quadSizeY = 32
      slider.imageWidth = 96
      slider.imageHeight = 32
      slider.quads = {
        left =  love.graphics.newQuad( 0, 0, slider.quadSizeX, slider.quadSizeY, slider.imageWidth, slider.imageHeight),
        mid =   love.graphics.newQuad(32, 0, slider.quadSizeX, slider.quadSizeY, slider.imageWidth, slider.imageHeight),
        right = love.graphics.newQuad(64, 0, slider.quadSizeX, slider.quadSizeY, slider.imageWidth, slider.imageHeight),
      }

function slider:internalSetup()
  self.type = "slider"
  self.objects = {}
  self.scrollY = 0
  self.canvas = love.graphics.newCanvas(self.w, self.h)
end

function slider:defineBounds()
  self.value = 1
  self.minValue = 1
  self.maxValue = 8
  self.incValue = 1
end

function slider:setup()
  local btnW = self.quadSize
  local tCol = engine.system.vars.user.uiText
  if self.image ~= nil then
    local image = self.image
    self:setBackground(image.."_back")
    self:setHoverColour(image.."_back_hover")
    self:setTextColour(tCol)
    self:add("val_dn", dnBtn:new("", 0, 0, btnW, btnW, image.."_button_left", image.."_button_left_hover", tCol))
    self:add("val_up", upBtn:new("", self.w - btnW, 0, btnW, btnW, image.."_button_right", image.."_button_right_hover", tCol))
    self:add("val_sl", slBtn:new(self:getValue(), btnW, 0, btnW, btnW, image.."_button", image.."_button_hover", tCol))
  else
    self:setBackground(bCol)
    self:setHoverColour(hCol)
    self:setTextColour(tCol)
    self:add("val_dn", dnBtn:new("-", 1, 0, btnW, btnW, self.bCol, self.hCol, tCol))
    self:add("val_up", upBtn:new("+", self.w - btnW + 1, 0, btnW, h, self.bCol, self.hCol, tCol))
    self:add("val_sl", slBtn:new("_", 1 + btnW, 0, btnW, h, self.bCol, self.hCol, tCol))
  end
  self:defineBounds()
end

function slider:setLimits(min, max, inc)
  self.minValue, self.maxValue, self.incValue = min, max, inc
end

function slider:getLimits()
  return self.minValue, self.maxValue, self.incValue
end

function slider:setValue(val)
  if val < self.minValue or val > self.maxValue then return end
  self.value = val
  self:getValuePosition()
  self:onValueChange()
  if self.dataLookup ~= nil then
    self.objects.val_sl:setLabel(self.dataLookup[val])
  else
    self.objects.val_sl:setLabel(val)
  end
  self.isDirty = true
end

function slider:onValueChange() end

function slider:getValue()
  if self.dataLookup ~= nil then
    return self.dataLookup[self.value]
  end
  return self.value
end

function slider:decrease()
  if self.value - self.incValue >= self.minValue then
    self:setValue(self.value - self.incValue)
  end
end

function slider:increase()
  if self.value + self.incValue <= self.maxValue then
    self:setValue(self.value + self.incValue)
  end
end

function slider:getPositionValue(x, y)
  if self.isHovered then
    local w, h = self:getSize()
    local minx = self.quadSize
    local maxx = w - (minx * 2)
    local minVal, maxVal, inc = self:getLimits()
    local notches = ((maxVal - minVal) / inc) + 1
    local notchWidth = (maxx - minx) / notches
    local notchPos = math.floor(x / notchWidth)
    self:setValue(notchPos * inc)
  end
end

function slider:getValuePosition()
  local val = self.value
  local w, h = self:getSize()
  local minx = self.quadSize
  local maxx = w - (minx * 2)
  local minVal, maxVal, inc = self:getLimits()
  local range = maxVal - minVal
  local notches = range / inc
  local currentNotch = ((self.value - minVal) / inc)
  local notchWidth = (maxx - minx) / notches
  local xPos = minx + (currentNotch * notchWidth)
  if self.objects.val_sl ~= nil then
    self.objects.val_sl:setPosition(xPos, 0)
  end
end

function slider:onScroll(s)
  if s > 0 then
    self:increase()
  else
    self:decrease()
  end
end

function slider:tileBackground()
  local image = self.image
  local x, y = 0, 0
  local w, h = self:getSize()
  local size = self.quadSizeX
  local scaleX = (w - (2 * self.quadSizeX)) / self.quadSizeX
  local scaleY = h / self.quadSizeY
  love.graphics.setColor(engine.system.vars.user.uiColour)
  if self.isHovered or self.isSelected and self.hoverImage ~= nil then
    image = self.hoverImage
  end
  love.graphics.draw(engine.assets[image], self.quads.mid, x + self.quadSizeX, y, 0, scaleX, scaleY)
  love.graphics.draw(engine.assets[image], self.quads.left, x, y)
  love.graphics.draw(engine.assets[image], self.quads.right, (x + w) - self.quadSizeX, y)
end

return slider