local tabBtn = engine.ui.button:extend()
      function tabBtn:onClick(x, y, b)
        self.box.objects[self.box.currentTab]:onClose()
        self.box.currentTab = self.label
        self.box.objects[self.box.currentTab]:onOpen()
        self.box.isDirty = true
      end
      function tabBtn:update(dt)
        self.isSelected = false
        if self.box.currentTab == self.label then
          self.isSelected = true
        end
      end

local tab = engine.ui.container:extend()

function tab:internalSetup()
  self.type = "Tab_Box"
  self.drawLayer = 1
  self.buttons = {}
  self.objects = {}
  self.currentTab = "none"
  self.tabCount = 0
  self.canvas = love.graphics.newCanvas(self.w, self.h)
  self.styleSheet = {
    
    buttons = {
      startX = 3,
      startY = 3,
      width = 64,
      height = 32,
      offsetX = 1,
      offsetY = 0,
      back = "standard_button",
      hover = "standard_button_hover",
      tCol = engine.system.vars.user.userText,
      tiled = true,
    },
    
    containers = {
      x = 3,
      y = 35,
      w = self.w - 6,
      h = self.h - 38,
      back = "standard_container",
      hover = "standard_container_hover",
      tCol = engine.system.vars.user.userText,
      tiled = true,
    },
    
  }
end

function tab:setStyleSheet(t)
  self.styleSheet = t
end

function tab:setScale(x, y)
  self.xScale = x
  self.yScale = y
  for k, v in pairs(self.buttons) do
    self.buttons[k]:setScale(x, y)
  end
  for k, v in pairs(self.objects) do
    self.objects[k]:setScale(x, y)
  end
  self.isDirty = true
end

function tab:addButton(label)
  self.tabCount = self.tabCount + 1
  local pos = self.tabCount - 1
  local btn = self.styleSheet.buttons
  local x = btn.startX + (btn.width * (pos * btn.offsetX))
  local y = btn.startY + (btn.height * (pos * btn.offsetY))
  self.buttons[label] = tabBtn:new(label, x, y, btn.width, btn.height, btn.back, btn.hover, btn.tCol, btn.tiled)
  if self.currentTab == "none" then
    self.currentTab = label
  end
  self.buttons[label].win = self.win
  self.buttons[label].box = self
  self.buttons[label].boxId = self.key
  self.buttons[label]:setup()
end

function tab:addTab(label, box)
  local con = self.styleSheet.containers
  self:addButton(label)
  self:add(label, box:new(label, con.x, con.y, con.w, con.h, con.back, con.hover, con.tCol, con.tiled))
end

function tab:mousePressed(x, y, b)
  local bool = self:checkHovered(x, y, b)
  if bool then
    self:deselectAll()
    if self.objects[self.currentTab]:mousePressed(x - self.x, y - self.y, b) then
      self.isGrabbed = false
      return true
    else
      self.grabx = x - self.x
      self.graby = y - self.y
      self.isGrabbed = true
      self.isDirty = true
      return true
    end
  end
  return false
end

function tab:mouseReleased(x, y, b)
  if self.buttons ~= nil then
    local xPos, yPos = self:getScaledPosition()
    local mx, my = x - xPos, y - yPos
    for k, v in pairs(self.buttons) do
      if self.buttons[k]:mouseReleased(mx, my, b) then
        self.isDirty = true
        return true
      end
    end
    self.objects[self.currentTab]:mouseReleased(mx, my, b)
  end
  return false
end

function tab:wheelMoved(mx, my, s)
  mx, my = mx - self.x, my - self.y
  if self.objects[self.currentTab]:wheelMoved(mx, my, s) then
    return true
  elseif self.isHovered then
    self:onScroll(s)
    return true
  end
  return false
end

function tab:updateMaster(dt, mx, my)
  self:checkHovered(mx, my)
  local xPos, yPos = self:getScaledPosition()
  local x, y = mx - xPos, my - yPos
  if self.buttons ~= nil then
    for k, _ in pairs(self.buttons) do
      self.buttons[k]:updateMaster(dt, x, y)
    end
  end
  self.objects[self.currentTab]:updateMaster(dt, x, y)
  
  self:redrawButtons()
  self.objects[self.currentTab]:draw()
  
  if self.isMoveable and self.isGrabbed then
    local newx = engine.mousex - self.grabx
    local newy = engine.mousey - self.graby
    self:setPosition(newx, newy)
  end
  self:update(dt, mx, my)
end

function tab:redrawButtons()
  for k, v in pairs(self.buttons) do
    self.buttons[k].isDirty = true
  end
end

function tab:draw()
  if self.isVisible then
    if self.isDirty then
      self:drawBackground()
      self:redrawButtons()
      self:redrawObjects()
    end
    
    for k, v in pairs(self.buttons) do
      self.buttons[k]:draw()
    end
    self.objects[self.currentTab]:draw()
    
    local lastCanvas = love.graphics.getCanvas()
    if self.box ~= nil then
      love.graphics.setCanvas(self.box.canvas)
    elseif self.win ~= nil then
      love.graphics.setCanvas(self.win.canvas)
    end
    engine.ui.colour.set("white")
    love.graphics.draw(self.canvas, self.x, self.y)
    love.graphics.setCanvas(lastCanvas)
  end
end

return tab