local manager = engine.ui.container:extend()

function manager:init()
  self.data = {
    isVisible = true,
    isHovered = false,
    isMoveable = false,
    isGrabbed = false,
    isSelected = true,
    isDirty = true,
  }
  self.label = "window_manager"
  self:setPosition(0, 0)
  self:setSize(engine.screenWidth, engine.screenHeight)
  self:internalSetup()
end

function manager:internalSetup()
  self.type = "window_manager"
  self.drawOrder = {}
  self.objects = {}
  dbg.add(self.type.." UI object '"..self.label.."' created.", "Instance "..engine.currentState)
end

function manager:add(key, object, ...)
  if self.objects == nil then self.objects = {} end
  self.objects[key] = object
  self.objects[key].manager = self
  self.objects[key].key = key
  self.objects[key]:setup(...)
  table.insert(self.drawOrder, 1, key)
  dbg.add(object.label.." window created.", "Instance "..engine.currentState)
end

function manager:closeWindow(key)
  dbg.add("Closing "..key..".", "WINDOW MANAGER")
  if self.objects[key] == nil then
    dbg.add(key.." does not exist.", "WINDOW MANAGER")
    return
  end
  self.objects[key]:onClose()
  self.objects[key] = nil
  local pos = self:getDrawLayer(key)
  dbg.add("Removing draw object "..pos.." "..key..".", "WINDOW MANAGER")
  table.remove(self.drawOrder, pos)
end

function manager:getDrawLayer(key)
  for i = 1, #self.drawOrder do
    if self.drawOrder[i] == key then
      return i
    end
  end
  return false
end

function manager:reorder(id)
  local value = self.drawOrder[id]
  table.remove(self.drawOrder, id)
  table.insert(self.drawOrder, 1, value)
end

function manager:windowExists(label)
  if self.objects[label] ~= nil then return true end
  return false
end

function manager:selectWindow(key)
  self:reorder(self:getDrawLayer(key))
end

function manager:mousePressed(x, y, b)
  if self.drawOrder[1] ~= nil then
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]]:mousePressed(x, y, b) then
        self:reorder(i)
        return true
      end
    end
  end
  return false
end

function manager:mouseReleased(x, y, b)
  if self.drawOrder[1] ~= nil then
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]]:mouseReleased(x, y, b) then
        return true
      end
    end
  end
  return false
end

function manager:wheelMoved(x, y, s)
  if self.drawOrder ~= nil then
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]]:wheelMoved(x, y, s) then
        return true
      end
    end
  end
  return false
end

function manager:updateMaster(dt, mx, my)
  if mx == nil then
    mx, my = engine.mousex, engine.mousey
  end
  if self.drawOrder ~= nil then
    local hoverDetected = false
    for i = 1, #self.drawOrder do
      if self.objects[self.drawOrder[i]] ~= nil then
        self.objects[self.drawOrder[i]]:updateMaster(dt, mx, my)
        if self.objects[self.drawOrder[i]].isHovered then
          if hoverDetected then
            self.objects[self.drawOrder[i]].isHovered = false
            self.objects[self.drawOrder[i]].isDirty = false
            if self.objects[self.drawOrder[i]].objects ~= nil then
              for k, v in pairs(self.objects[self.drawOrder[i]].objects) do
                self.objects[self.drawOrder[i]].objects[k].isHovered = false
                self.objects[self.drawOrder[i]].objects[k].isDirty = false
              end
            end
          else
            hoverDetected = true
          end
        end
      end
    end
  end
  self:update(dt, mx + self.x, my + self.y)
  --if self.drawOrder[1] ~= nil then
    --for i = #self.drawOrder, 1, -1 do
      --if self.objects[self.drawOrder[i]].isVisible then
        --self.objects[self.drawOrder[i]]:draw()
      --end
    --end
  --end
end

function manager:draw()
  if self.drawOrder[1] ~= nil then
    love.graphics.setColor(255,255,255,255)
    for i = #self.drawOrder, 1, -1 do
      if self.objects[self.drawOrder[i]].isVisible then
        local x = self.objects[self.drawOrder[i]].x
        local y = self.objects[self.drawOrder[i]].y
        love.graphics.draw(self.objects[self.drawOrder[i]].canvas, x, y, 0, self.objects[self.drawOrder[i]].xScale, self.objects[self.drawOrder[i]].yScale)
      end
    end
  end
end

return manager