dbg = require "/Lib/system/debug"
utf8 = require("utf8")
font = love.graphics.newFont()
UPDATE_PLAYER_DATA = false
local fs = love.filesystem
local gr = love.graphics

game = {}
engine = {
  version = "0.0.2",
  currentState = "main_menu",
  currentSong = "Sneaky_Adventure",
  currentFont = "good_times_rg",
  running = true,
  mousex = 1,
  mousey = 1,
  mouseButton = 0,
  screenWidth = 1280,
  screenHeight = 720,
  state = {},
  ui = {},
}

function math.round(n, deci)
  deci = 10^(deci or 0)
  return math.floor(n*deci+.5)/deci
end

function engine.state.set(label, ...)
  engine.lastState = engine.currentState
  if engine.state[engine.lastState].exit ~= nil then
    engine.state[engine.lastState].exit()
  end
  
  engine.currentState = label
  if not engine.state[label].initialized then
    if engine.state[label].startup ~= nil then
      engine.state[label].startup(...)
    end
    engine.state[label].initialized = true
  end
  
  if engine.state[label].enter ~= nil then
    engine.state[label].enter()
  end
end

local function getFile(tab, path, folder, file)
  if tab[folder] == nil then
    tab[folder] = {}
  end
  local ext = string.sub(file, #file - 3, #file)
  file = string.sub(file, 1, #file - 4)
  local filePath = path.."/"..folder.."/"..file
  if ext == ".lua" then
    tab[folder][file] = require(path.."."..folder.."."..file) --Those "."'s are for you Bartbes :P
  elseif ext == ".png" then
    tab[folder][file] = gr.newImage(filePath..ext)
  elseif ext == ".sha" then
    tab[folder][file] = gr.newShader(filePath..ext)
  elseif ext == ".mp3" then
    tab[folder][file] = love.audio.newSource(filePath..ext, "static")
  elseif ext == ".ttf" then
    tab[folder][file] = love.graphics.newFont(filePath..ext, 10)
  end
  if tab[folder][file] ~= nil then
    dbg.add("engine."..folder.."."..file, "Loaded")
  end
end

local function getFiles(tab, path, folder)
  local filePath = path
  if folder == nil then
    folder = ""
  else
    filePath = path.."/"..folder
  end
  dbg.add("Checking "..filePath, "Loading")
  local data = fs.getDirectoryItems(filePath)
  for _, file in pairs(data) do
    if fs.isDirectory(filePath.."/"..file) then
      dbg.add("Directory found "..file, "Loading")
      if folder ~= "" then
        getFiles(tab, filePath, file)
      else
        getFiles(tab, path, file)
      end
    else
      getFile(tab, path, folder, file)
    end
  end
end

function love.load(arg)
  if arg[#arg] == "-debug" then require("mobdebug").start() end
  love.window.maximize()
  math.randomseed(os.time())
  fs.createDirectory("/Mods")
  getFiles(engine, "Lib")
  getFiles(engine, "Data")
  getFiles(engine, "Mods")
  --love.graphics.setFont(engine.font[engine.currentFont])
  engine.state.set(engine.currentState)
end

function love.resize(...)
  if engine.state[engine.currentState].resize ~= nil then
    engine.state[engine.currentState].resize(...)
  end
end

function love.keypressed(...)
  if engine.state[engine.currentState].keyPressed ~= nil then
    engine.state[engine.currentState].keyPressed(...)
  end
end

function love.textinput(...)
  if engine.state[engine.currentState].textInput ~= nil then
    engine.state[engine.currentState].textInput(...)
  end
end

function love.mousepressed(...)
  if engine.state[engine.currentState].mousePressed ~= nil then
    engine.state[engine.currentState].mousePressed(...)
  end
end

function love.mousereleased(...)
  if engine.state[engine.currentState].mouseReleased ~= nil then
    engine.state[engine.currentState].mouseReleased(...)
    engine.mouseButton = 0
  end
end

function love.wheelmoved(x, y)
  if engine.state[engine.currentState].wheelMoved ~= nil then
    engine.state[engine.currentState].wheelMoved(engine.mousex, engine.mousey, y)
  end
end

function love.update(dt)
  if engine.state[engine.currentState].update ~= nil then
    engine.mouseButton = 0
    if love.mouse.isDown(1) then
      engine.mouseButton = 1
    elseif love.mouse.isDown(2) then
      engine.mouseButton = 2
    end
    engine.mousex, engine.mousey = love.mouse.getPosition()
    local w, h = love.graphics.getDimensions()
    if w ~= engine.screenWidth or h ~= engine.screenHeight then
      engine.screenWidth, engine.screenHeight = w, h
      love.resize(w, h)
    end
    engine.state[engine.currentState].update(dt)
    collectgarbage()
  end
end

function love.draw()
  if engine.state[engine.currentState].draw ~= nil then
    engine.state[engine.currentState].draw()
  end
end

function love.quit()
  game = nil
  engine = nil
  dbg = nil
  collectgarbage()
end